Package: emln
Type: Package
Title: Organize, Handle, and Explore Ecological Multilayer Networks
Version: 1.2.0
Date: 2026-05-13
Authors@R: c(
    person("Noa", "Frydman", role = "aut"),
    person("Shai", "Pilosof", role = c("aut", "fnd"), email = "pilos@bgu.ac.il"),
    person("Shirly", "Freilikhman", role = "aut"),
    person("Geut", "Galai", role = "cre", email = "geutg@post.bgu.ac.il")
    )
URL: https://emln.ecomplab.com/
BugReports: https://github.com/Ecological-Complexity-Lab/emln/issues
Description: Data and analysis of ecological multilayer networks, including standardization of data structures and functions to convert between them. Includes an interactive multilayer network visualizer (beta, paper forthcoming), and a collection of 78 empirical ecological multilayer network datasets. This work was supported by research grant ISF (Israel Science Foundation) 1281/20 to Shai Pilosof. Noa Frydman (2023) <doi:10.1111/2041-210X.14225>. 
License: CC BY 4.0
Encoding: UTF-8
LazyData: true
LazyDataCompression: xz
Depends: R (>= 4.1.0)
Imports: magrittr, stringr, igraph, tibble, dplyr, purrr, tidyr,
        Matrix, DT, hablar, rlang
Suggests: bipartite, httpuv, testthat (>= 3.0.0)
RoxygenNote: 7.3.2
Config/testthat/edition: 3
NeedsCompilation: no
Packaged: 2026-05-13 09:16:40 UTC; geutgalai
Author: Noa Frydman [aut],
  Shai Pilosof [aut, fnd],
  Shirly Freilikhman [aut],
  Geut Galai [cre]
Maintainer: Geut Galai <geutg@post.bgu.ac.il>
Repository: CRAN
Date/Publication: 2026-05-13 11:00:02 UTC
Built: R 4.6.0; ; 2026-05-13 11:54:19 UTC; unix

# Generated by roxygen2: do not edit by hand

export(create_monolayer_network)
export(create_multilayer_network)
export(get_igraph)
export(get_sam)
export(list_to_matrix)
export(load_emln)
export(matrix_to_list_bipartite)
export(matrix_to_list_unipartite)
export(multilayer_to_csv)
export(multilayer_to_json)
export(plot_multilayer)
export(search_emln)
export(view_emln)
import(DT)
import(dplyr)
import(tibble)
import(tidyr)
importFrom(Matrix,Matrix)
importFrom(hablar,retype)
importFrom(igraph,V)
importFrom(igraph,as_adjacency_matrix)
importFrom(igraph,as_biadjacency_matrix)
importFrom(igraph,degree)
importFrom(igraph,delete_vertex_attr)
importFrom(igraph,graph.incidence)
importFrom(igraph,graph_from_adjacency_matrix)
importFrom(igraph,graph_from_biadjacency_matrix)
importFrom(igraph,graph_from_data_frame)
importFrom(igraph,is_directed)
importFrom(magrittr,"%<>%")
importFrom(magrittr,"%>%")
importFrom(purrr,compact)
importFrom(rlang,.data)
importFrom(stringr,str_detect)
importFrom(stringr,str_to_title)

![](https://img.shields.io/badge/R--CMD--CHK-Passed-green.svg)  [![Project Status: Active - The project has reached a stable, usable state and is being actively developed.](https://www.repostatus.org/badges/latest/active.svg)](https://www.repostatus.org/#active) ![](https://img.shields.io/badge/version-1.2.0-blue.svg) [![CRAN status](https://www.r-pkg.org/badges/version/emln)](https://CRAN.R-project.org/package=emln)  [![](https://img.shields.io/badge/lifecycle-stable-brightgreen.svg)](https://lifecycle.r-lib.org/articles/stages.html#stable) [![](https://img.shields.io/badge/doi-10.1111/2041--210X.14225-orange.svg)](https://doi.org/10.1111/2041-210X.14225) [![](https://img.shields.io/badge/MiRA-arXiv%3A2605.09597-b31b1b.svg)](https://doi.org/10.48550/arXiv.2605.09597)

# :wave: About
**This repository contains the code for the R package EMLN.** EMLN standardizes workflows for creating, storing, and converting multilayer network data, and ships with a collection of empirical ecological multilayer datasets ready for analysis. It also provides interactive, browser-based visualization through its integration with **[MiRA](https://mira.ecomplab.com/)**, launched directly from R via `plot_multilayer()`. Although designed with ecological data in mind, EMLN is flexible and can handle data from other research domains.

# :page_facing_up: Paper and citing
Frydman N, Freilikhman S, Talpaz I, Pilosof S. **Practical guidelines and the EMLN R package for handling ecological multilayer networks**. Methods in Ecology and Evolution. 2023. [DOI:10.1111/2041-210X.14225](https://besjournals.onlinelibrary.wiley.com/doi/10.1111/2041-210X.14225). Please cite the paper when implementing the guidelines we describe or when using the package, this helps us a lot!

# :package: Installation
EMLN is available on CRAN. installation is as follows:
```R
install.packages("emln")
```


# :globe_with_meridians: Website
Detailed explanations on workflows accompanied by examples for handling monolayer and multilayer data using emln are in: [emln.ecomplab.com](https://emln.ecomplab.com/).

# :bar_chart: Interactively visualizaing multilayer networks
EMLN integrates **[MiRA](https://mira.ecomplab.com/)** (Multilayer Interactive Rendering Application), a browser-based, installation-free visualizer launched from R via `plot_multilayer()`, or by exporting with `multilayer_to_json()` / `multilayer_to_csv()`. MiRA offers seven complementary modes — Network (3D), Map, Grid View, Layer View, Meta-Network, Dashboard, and Data — with interactive rotation, filtering, color/size mapping, and bipartite support, plus nine bundled empirical datasets.

If you use MiRA in your published research, please cite the MiRA preprint:

Nehoray SM, Bloch Y, Pilosof S (2026). **Interactively visualizing biological multilayer networks using MiRA**. *arXiv*:2605.09597 [cs.SI]. [https://doi.org/10.48550/arXiv.2605.09597](https://doi.org/10.48550/arXiv.2605.09597)

# MiRA — Multilayer Interactive Rendering Application

[![Project Status: Active](https://www.repostatus.org/badges/latest/active.svg)](https://www.repostatus.org/#active)
[![Lifecycle: stable](https://img.shields.io/badge/lifecycle-stable-brightgreen.svg)](https://lifecycle.r-lib.org/articles/stages.html)
[![License: CC BY-NC-SA 4.0](https://img.shields.io/badge/License-CC%20BY--NC--SA%204.0-lightgrey.svg)](https://creativecommons.org/licenses/by-nc-sa/4.0/)
[![R package: emln](https://img.shields.io/badge/R%20package-emln-blue.svg)](https://github.com/Ecological-Complexity-Lab/emln)
[![arXiv](https://img.shields.io/badge/arXiv-2605.09597-b31b1b.svg)](https://doi.org/10.48550/arXiv.2605.09597)

An interactive, browser-based visualization tool for multilayer networks. While developed with biology in mind, MiRA can render any multilayer network. Available as a standalone web app and also fully integrated into the [`emln`](https://github.com/Ecological-Complexity-Lab/emln) R package.


## 📝 Citation
If you use MiRA to explore your networks as part of your published research or to produce figures, please cite our preprint:

Nehoray SM, Bloch Y and Pilosof S (2026). **Interactively visualizing biological multilayer networks using MiRA**. _arXiv_:2605.09597 [cs.SI]. [https://doi.org/10.48550/arXiv.2605.09597](https://doi.org/10.48550/arXiv.2605.09597)

<details>
<summary>BibTeX</summary>

```bibtex
@article{Nehoray2026,
  title        = {Interactively visualizing biological multilayer networks using {MiRA}},
  author       = {Nehoray, Shir Miryam and Bloch, Yuval and Pilosof, Shai},
  journal      = {arXiv [cs.SI]},
  year         = {2026},
  eprint       = {2605.09597},
  archivePrefix= {arXiv},
  primaryClass = {cs.SI},
  doi          = {10.48550/arXiv.2605.09597},
  url          = {https://doi.org/10.48550/arXiv.2605.09597}
}
```
</details>

## 📄 Documentation and help

Full feature reference, data format guide, and controls reference: **[mira.ecomplab.com/docs/manual.html](https://mira.ecomplab.com/docs/manual.html)** (also accessible via the **?** button inside the app).

## ✨ Visualization modes

- **Network Mode** — 3D stacked-layer canvas with rotate, pan, drag, hover, and selection.
- **Map Mode** — layers placed on an interactive Leaflet world map using geographic coordinates.
- **Grid View** — small-multiples matrix layout: each layer in its own panel, all sharing the same color and selection state. Ideal for comparing intralayer structure across many layers simultaneously.
- **Layer View** — meta-graph where each layer is a force-directed bubble with micro-graph previews and side-by-side comparison panels.
- **Meta-Network Mode** — aggregated single-layer view of cross-layer connectivity.
- **Dashboard Mode** — analytics panels: KPI cards, per-layer charts, presence matrix, Jaccard similarity heatmaps, degree distributions, link weight distributions, node participation, and bipartite set-size ratios.
- **Data Mode** — tabular inspection and subsetting of nodes, links, and layer attributes.

### Best visualization practices:
Multilayer networks are inherently complex. Unlike monolayer networks, which can be visualized in 2D, the 3D representation of multilayer networks adds visual complexity. Therefore, while it is possible to load large networks, visualizing small to medium-sized networks is more effective. The more layers and interlayer links present, the more entangled the visualization will be. Furthermore, visualizations heavily loaded with links will be slower to interact with (e.g., dragging and navigating the network). **MiRA** is designed to help you manage this complexity — for example, by allowing you to set thresholds on visualized links, apply different layouts, and by using summary visualizations such as *Layer View*, *Grid View*, and *Meta-Network* modes, and by filtering layers, nodes, and links in Data mode. For detailed guidance on choosing the right mode and managing visual complexity, see the [Visualization Guidelines](https://mira.ecomplab.com/docs/manual.html#guidelines) section of the manual.

## 🚀 Loading MiRA

### 🌐 Online
Use the live version at [https://mira.ecomplab.com/](https://mira.ecomplab.com/).

### 💻 Locally
The app uses ES Modules and must be served over HTTP — it cannot be opened via `file://`.

```bash
git clone https://github.com/Ecological-Complexity-Lab/MiRA.git
cd MiRA
python3 -m http.server 8000
# Open http://localhost:8000
```

## 🗂️ Built-in Example Datasets

Nine empirical multilayer networks are bundled for immediate exploration, from six biological domains, covering pollination, host–parasite interactions, seed dispersal, gene recombination, human disease, brain connectivity, plasmid sharing, and protein–protein interactions. See the full list and references in the [manual](https://mira.ecomplab.com/docs/manual.html).

## 📥 Import Your Own Multilayer Network

- **JSON** — native format.
- **CSV** — extended edge list + optional layer and node attribute files.

See the [manual](https://mira.ecomplab.com/docs/manual.html#data-import) for schemas.

## 🔗 Integration with the R Package `emln`

MiRA is bundled inside the [`emln`](https://github.com/Ecological-Complexity-Lab/emln) R package. Launch directly from R with:

```r
plot_multilayer(net, bipartite = TRUE, directed = FALSE)
```

Or export to JSON/CSV for manual loading:

```r
multilayer_to_json(net, file = "my_network.json", bipartite = TRUE)
multilayer_to_csv(net, path = "my_network/")
```

## App development process
We developed the MiRA codebase using an AI-assisted workflow. Initial scaffolding and iterative feature implementation were carried out using Claude Code (Anthropic), an agentic AI programming tool, with Claude Sonnet~4.6 used for routine implementation tasks and Claude Opus~4 used for complex architectural decisions and refactoring. Human oversight was applied throughout: all AI-generated code was reviewed, debugged, and revised. We paid particular attention to calculation of properties, domain-specific logic, including bipartite layout computation, geographic placement of layers, multilayer data structure validation, and EMLN integration. The codebase was audited periodically to identify inconsistencies between behavior and documentation, to enforce modularity via refactoring, and to remove redundancy introduced during iterative development.

## Feedback and issues

Feedback and bug reports are welcome at [GitHub Issues](https://github.com/Ecological-Complexity-Lab/MiRA/issues).

## 📄 License

MiRA is licensed under the [Creative Commons Attribution-NonCommercial-ShareAlike 4.0 International License](https://creativecommons.org/licenses/by-nc-sa/4.0/).

[![License: CC BY-NC-SA 4.0](https://img.shields.io/badge/License-CC%20BY--NC--SA%204.0-lightgrey.svg)](https://creativecommons.org/licenses/by-nc-sa/4.0/)

You are free to share and adapt this work for **non-commercial purposes**, provided you give appropriate credit and distribute any derivative works under the same license.

#' Search networks
#'
#'@name search_emln
#'
#' @description Search the database for networks depending on search parameters specified below.
#'
#' @param ecological_network_type One of: Seed-Dispersal, Pollination, Food-Web, Host-Parasite, Multiple, Plant-Herbivore, Anemone-Fish, Plant-Ant
#' @param multilayer_network_type Environment, Temporal, Multiplex, Perturbation, Spatial
#' @param state_nodes TRUE/FALSE
#' @param weighted TRUE/FALSE
#' @param directed TRUE/FALSE
#' @param interlayer TRUE/FALSE
#' @param layer_number_minimum The minimum number of layers wanted in the network
#' @param node_number_minimum The minimum number of layers wanted in the network
#'
#'
#' @return A tibble containing the network id, and all the search arguments for each of the networks that are compatible with the search.
#'
#'
#' @examples
#' # See examples in:
#' # https://emln.ecomplab.com/data.html#Browsing_the_data
#'
#' # Generates a tibble of all the networks in the package that are pollination networks
#' search_emln(ecological_network_type = 'Plant-Ant')
#'
#' # Generates a tibble of all the networks in the package that have state nodes and are temporal
#' search_emln(multilayer_network_type = 'Temporal', state_node = TRUE)
#'
#' @export search_emln
#' @import dplyr
#' @importFrom stringr str_to_title
#' @importFrom purrr compact
#' @importFrom magrittr %>% %<>%
#' @importFrom rlang .data



search_emln <- function(ecological_network_type=NULL,multilayer_network_type=NULL,
                        state_nodes=NULL,node_number_minimum=NULL,layer_number_minimum=NULL,
                        weighted = NULL, interlayer = NULL, directed =NULL){


  #load the large tibble that contains all the descriptions for it to be filtered

  #desc <- loadRData('./data/descriptions.rda')
  desc <- descriptions


  #check which arguments were entered so I can loop over it.

  attributes_entered <- purrr::compact(as.list(match.call()))[-1]

  for (i in names(attributes_entered)) {
      if (i == 'ecological_network_type') {
        desc <- desc %>%
          dplyr::filter(.data$ecological_network_type == stringr::str_to_title(attributes_entered[['ecological_network_type']]))
        next
      } else if (i == 'multilayer_network_type') {
        desc <- desc %>%
          dplyr::filter(.data$multilayer_network_type == stringr::str_to_title(attributes_entered[['multilayer_network_type']]))
        next
      } else if (i == 'state_nodes') {
        desc <- desc %>%
          dplyr::filter(.data$state_nodes == attributes_entered[['state_nodes']])
        next
      } else if (i == 'node_number_minimum') {
        desc <- desc %>%
          dplyr::filter(.data$node_num >= attributes_entered[['node_number_minimum']])
        next
      } else if (i == 'layer_number_minimum') {
        desc <- desc %>%
          dplyr::filter(.data$layer_num >= attributes_entered[['layer_number_minimum']])
        next
      } else if (i == 'weighted') {
        desc <- desc %>%
          dplyr::filter(.data$weighted == attributes_entered[['weighted']])
        next
      } else if (i == 'directed') {
        desc <- desc %>%
          dplyr::filter(.data$directed == attributes_entered[['directed']])
        next
      } else if (i == 'interlayer') {
        desc <- desc %>%
          dplyr::filter(.data$interlayer == attributes_entered[['interlayer']])
        }
        next
      }

  #sort the tibble

  desc <- desc %>%
    dplyr::group_by(.data$network_id, .data$network_name)  %>%
    dplyr::arrange(.data$network_id, .data$network_name)

  if (nrow(desc) == 0) {return('There are no Networks with that combination of attributes')}

  return(desc)

  }

test_that("search_emln with no arguments returns a data frame", {
  result <- search_emln()
  expect_true(is.data.frame(result))
  expect_gt(nrow(result), 0)
})

test_that("search_emln result contains required columns", {
  result <- search_emln()
  expect_true(all(c("network_id", "network_name", "ecological_network_type",
                    "multilayer_network_type") %in% names(result)))
})

test_that("search_emln result is sorted by network_id", {
  result <- search_emln()
  expect_false(is.unsorted(result$network_id))
})

test_that("search_emln filters by ecological_network_type", {
  result <- search_emln(ecological_network_type = "Pollination")
  expect_true(is.data.frame(result))
  expect_true(all(result$ecological_network_type == "Pollination"))
})

test_that("search_emln filters by multilayer_network_type", {
  result <- search_emln(multilayer_network_type = "Temporal")
  expect_true(is.data.frame(result))
  expect_true(all(result$multilayer_network_type == "Temporal"))
})

test_that("search_emln filters by weighted", {
  result <- search_emln(weighted = TRUE)
  if (is.data.frame(result) && nrow(result) > 0) {
    expect_true(all(result$weighted == TRUE))
  }
})

test_that("search_emln filters by directed", {
  result <- search_emln(directed = TRUE)
  if (is.data.frame(result) && nrow(result) > 0) {
    expect_true(all(result$directed == TRUE))
  }
})

test_that("search_emln filters by interlayer", {
  result <- search_emln(interlayer = TRUE)
  if (is.data.frame(result) && nrow(result) > 0) {
    expect_true(all(result$interlayer == TRUE))
  }
})

test_that("search_emln filters by state_nodes", {
  result <- search_emln(state_nodes = TRUE)
  if (is.data.frame(result) && nrow(result) > 0) {
    expect_true(all(result$state_nodes == TRUE))
  }
})

test_that("search_emln filters by layer_number_minimum", {
  min_layers <- 5
  result <- search_emln(layer_number_minimum = 5)
  print(result)
  if (is.data.frame(result) && nrow(result) > 0) {
    expect_true(all(result$layer_num >= min_layers))
  } else { # got a string
    expect_true(FALSE, info = "Expected a data frame with at least one row.")
  }
})

test_that("search_emln filters by node_number_minimum", {
  min_nodes <- 10
  result <- search_emln(node_number_minimum = 10)
  if (is.data.frame(result) && nrow(result) > 0) {
    expect_true(all(result$node_num >= min_nodes))
  }
})

test_that("search_emln returns fewer networks with tighter filters", {
  all_networks     <- search_emln()
  spatial_networks <- search_emln(multilayer_network_type = "Spatial")
  expect_lt(nrow(spatial_networks), nrow(all_networks))
})

test_that("search_emln returns a character string when no networks match", {
  # Extremely restrictive combination that is unlikely to match anything
  result <- search_emln(
    ecological_network_type = "Pollination",
    directed = TRUE,
    interlayer = TRUE,
    layer_number_minimum = 200
  )
  # The function returns a character string when nrow == 0
  expect_true(is.character(result) ||
                (is.data.frame(result) && nrow(result) == 0))
})

#' Creates a monolayer network object.
#'
#' Automatically identifies if the input is a matrix or an edge list and creates
#' a \code{monolayer} object.
#'
#' @param x input data in the format of a matrix, edge list or an igraph object.
#' @param directed Is the network directed?
#' @param bipartite Is the network bipartite? Ignored when the input is an igraph object.
#' @param group_names For bipartite networks: name of the groups in the columns
#'   and rows, respectively (e.g., parasites and hosts).
#' @param node_metadata Following the igraph method of \code{graph_from_data_frame}.
#'   First column must be have names matching those in x.
#'
#' @return A \code{monolayer} object.
#'
#' @seealso \code{list_to_matrix, matrix_to_list_unipartite, monolayer}.
#'
#' @details Converts between edge list and matrix formats and creates a
#'   monolayer object. It a wrapper function for
#'   \code{list_to_matrix, matrix_to_list_unipartite, matrix_to_list_bipartite}. Node metadata can only be included with an edge list input.
#'
#' @examples
#' library(igraph)
#'
#' # A bipartite network from package bipartte
#' x <- create_monolayer_network(bipartite::memmott1999, bipartite = TRUE,
#' directed = FALSE, group_names = c('Animals','Plants'))
#'
#' # A bipartite network as an igraph object
#'
#' # Generate a random bipartite network in igraph
#' g <- igraph::sample_bipartite(10,16,p=0.3, type = 'gnp', directed = TRUE, mode = 'in')
#' V(g)$name <- letters # name the nodes
#' V(g)$gender <- sample(c('F','M'), size = 26, replace = TRUE) # Add a node attribute
#' E(g)$weight=runif(ecount(g)) # Add edge weights
#' plot(g, layout=layout.bipartite)
#' create_monolayer_network(g, group_names = c('Parasites','Hosts'))
#'
#'
#'
#'
#' @export
#' @import dplyr
#' @importFrom igraph graph.incidence V
#' @importFrom rlang .data

create_monolayer_network <- function(x, directed=NULL, bipartite=NULL, group_names=c('set_cols','set_rows'), node_metadata=NULL){

  # Input is a matrix
  if ('matrix'%in%class(x)){
    if(bipartite){
      message('Input: a bipartite matrix')
      out <- matrix_to_list_bipartite(x, group_names = group_names)
    } else {
      out <- matrix_to_list_unipartite(x, directed = directed)
    }
  }

  # Input is an edge list
  if ('data.frame'%in%class(x)){
    if (bipartite){message('Input: a bipartite edge list')}
    if (!bipartite){message('Input: an unipartite edge list')}
    out <- list_to_matrix(x, directed, bipartite, group_names, node_metadata = node_metadata)
  }

  # Input is an igrpah object
  if('igraph'%in%class(x)){
    message('Input: an igraph object:')
    message(x)
    g <- x
    mode <- ifelse(igraph::is_bipartite(g),'B','U')

    node_list <- igraph::as_data_frame(g, what = 'vertices') %>%
      mutate(node_id=1:igraph::vcount(g)) %>%
      rename(node_name=name) %>%
      select("node_id", "node_name", everything())

    if(mode=='B'){
      node_list$node_group <- NA
      node_list$node_group[node_list$type==TRUE] <- group_names[1]
      node_list$node_group[node_list$type==FALSE] <- group_names[2]
      mat <- igraph::as_biadjacency_matrix(g,names = TRUE, attr = 'weight', sparse = FALSE)
    } else {
      mat <- igraph::as_adjacency_matrix(g, type = 'both', attr = 'weight', sparse = FALSE)
    }

    out <- list(mode=mode,
                directed=igraph::is_directed(g),
                nodes=node_list,
                mat= mat,
                edge_list=as_tibble(igraph::as_data_frame(g, 'edges')),
                igraph_object=g)
  }

  # Add additional node attributes
  if (!is.null(node_metadata)){
    out$nodes %<>% left_join(node_metadata)
  }
  # Create an edge list with IDs instead of node names
  out$edge_list_ids <-
    out$edge_list %>%
    left_join(out$nodes, by=c('from' = 'node_name')) %>%
    left_join(out$nodes, by=c('to' = 'node_name')) %>%
    dplyr::select(-"from", -"to") %>%
    dplyr::select(from="node_id.x", to="node_id.y", "weight")
  class(out) <- "monolayer"
  return(out)
}

mat_unip <- matrix(
  c(0, 1, 0,
    0, 0, 1,
    0, 0, 0),
  nrow = 3, byrow = TRUE,
  dimnames = list(c("A", "B", "C"), c("A", "B", "C"))
)

mat_bip <- matrix(
  c(1, 0, 1,
    0, 1, 1),
  nrow = 2, byrow = TRUE,
  dimnames = list(c("Plant1", "Plant2"), c("Bee1", "Bee2", "Bee3"))
)

el_unip <- data.frame(
  from   = c("A", "B", "C"),
  to     = c("B", "C", "A"),
  weight = c(1, 2, 3)
)

el_bip <- data.frame(
  from   = c("Plant1", "Plant1", "Plant2"),
  to     = c("Bee1", "Bee2", "Bee2"),
  weight = c(1, 2, 3)
)

test_that("create_monolayer_network handles unipartite matrix input", {
  result <- create_monolayer_network(mat_unip, directed = TRUE, bipartite = FALSE)
  expect_s3_class(result, "monolayer")
  expect_equal(result$mode, "U")
})

test_that("create_monolayer_network handles bipartite matrix input", {
  result <- create_monolayer_network(mat_bip, directed = FALSE, bipartite = TRUE,
                                     group_names = c("Bees", "Plants"))
  expect_s3_class(result, "monolayer")
  expect_equal(result$mode, "B")
})

test_that("create_monolayer_network handles unipartite edge list input", {
  result <- create_monolayer_network(el_unip, directed = TRUE, bipartite = FALSE)
  expect_s3_class(result, "monolayer")
  expect_equal(result$mode, "U")
})

test_that("create_monolayer_network handles bipartite edge list input", {
  result <- create_monolayer_network(el_bip, directed = FALSE, bipartite = TRUE)
  expect_s3_class(result, "monolayer")
  expect_equal(result$mode, "B")
})

test_that("create_monolayer_network returns edge_list_ids with integer IDs", {
  result <- create_monolayer_network(mat_unip, directed = TRUE, bipartite = FALSE)
  expect_true("edge_list_ids" %in% names(result))
  expect_true(all(c("from", "to", "weight") %in% names(result$edge_list_ids)))
  expect_true(is.numeric(result$edge_list_ids$from))
  expect_true(is.numeric(result$edge_list_ids$to))
})

test_that("create_monolayer_network handles igraph object input", {
  g <- igraph::graph_from_adjacency_matrix(mat_unip, mode = "directed", weighted = TRUE)
  result <- create_monolayer_network(g)
  expect_s3_class(result, "monolayer")
  expect_equal(result$mode, "U")
})

test_that("create_monolayer_network node_metadata is joined into nodes table", {
  meta <- data.frame(node_name = c("A", "B", "C"), trait = c("x", "y", "z"),
                     stringsAsFactors = FALSE)
  result <- suppressMessages(
    create_monolayer_network(el_unip, directed = TRUE, bipartite = FALSE,
                             node_metadata = meta)
  )
  expect_true("trait" %in% names(result$nodes))
  expect_equal(nrow(result$nodes), 3)
})

test_that("create_monolayer_network directed flag propagates correctly", {
  result_dir <- create_monolayer_network(mat_unip, directed = TRUE, bipartite = FALSE)
  result_und <- create_monolayer_network(
    matrix(c(0, 1, 1, 1, 0, 1, 1, 1, 0), nrow = 3,
           dimnames = list(c("A", "B", "C"), c("A", "B", "C"))),
    directed = FALSE, bipartite = FALSE
  )
  expect_true(result_dir$directed)
  expect_false(result_und$directed)
})

#'
#' @docType data
#' @title emln78_temporal_zackenberg_2013
#' 
#' @description Network Description:
#' 
#' | network_id | multilayer_network_type | ecological_network_type | state_nodes |
#' |:----------:|:-----------------------:|:-----------------------:|:-----------:|
#' |     78     |        Temporal         |       Pollination       |    FALSE    |
#' 
#' @format NULL
#' @usage NULL
#' @source plos
#' @source https://journals.plos.org/plosone/article?id=10.1371/journal.pone.0081694
#' @md
#' @keywords internal
'emln78_temporal_zackenberg_2013'

#'
#' @docType data
#' @title emln71_temporal_perturbation_kaiser_bunbury_2009
#' 
#' @description Network Description:
#' 
#' | network_id |  multilayer_network_type   | ecological_network_type | state_nodes |
#' |:----------:|:--------------------------:|:-----------------------:|:-----------:|
#' |     71     | Temporal    , Perturbation |       Pollination       |    FALSE    |
#' 
#' @format NULL
#' @usage NULL
#' @source Interaction_Web_DataBase
#' @source Web_of_Life
#' @source http://www.ecologia.ib.usp.br/iwdb/html/kaiser-bunbury_2009.html
#' @source http://www.web-of-life.es/map.php
#' @md
#' @keywords internal
'emln71_temporal_perturbation_kaiser_bunbury_2009'

#' Plot a multilayer network in the browser
#'
#' Converts an EMLN \code{multilayer} object to JSON and opens MiRA, the
#' multilayer interactive R-network app, in the default web browser.
#'
#' @param multilayer A multilayer object (created by \code{create_multilayer_network}
#'   or \code{load_emln}).
#' @param bipartite Logical. Whether the network is bipartite. Defaults to
#'   \code{FALSE}. Bipartite is not auto-detected. When \code{TRUE}, the nodes
#'   table must contain a \code{node_type} (or legacy \code{node_group})
#'   column with exactly two distinct values across the network.
#' @param setA_type Optional character. The \code{node_type} value to render
#'   as Set A (top row) in MiRA's bipartite layout. By ecological convention
#'   this is the higher trophic level (e.g. \code{"pollinator"},
#'   \code{"parasite"}, \code{"disperser"}). Only used when
#'   \code{bipartite = TRUE}. Forwarded to \code{multilayer_to_json}.
#' @param directed Logical or NULL. Whether intralayer links are directed.
#'   If NULL (default), auto-detected by checking intralayer edge symmetry.
#' @param directed_interlayer Logical or NULL. Whether interlayer links are
#'   directed. If NULL (default), inherits from \code{directed}.
#' @param port Integer. Port for the local HTTP server. Default is 8080.
#' @param mira_path Character. Path to the MiRA web-app directory. If NULL
#'   (default), uses the copy bundled with the package
#'   (\code{system.file("MiRA", package = "emln")}).
#' @param browser Choose the specific browser to open MiRA in. Defaults to
#'   the system default via \code{getOption("browser")}. Can be a browser
#'   name (e.g. "chrome", "firefox") or a command
#'   (e.g. "open -a 'Google Chrome'").
#'
#' @note
#' MiRA (Multilayer Interactive Rendering Application) is a browser-based
#' interactive visualization tool for multilayer networks, available at
#' \url{https://mira.ecomplab.com/}.
#'
#' If you use MiRA in published research, please cite:
#'
#' Nehoray SM, Bloch Y, Pilosof S (2026). Interactively visualizing biological
#' multilayer networks using MiRA. \emph{arXiv}:2605.09597 [cs.SI].
#' \doi{doi:10.48550/arXiv.2605.09597}
#'
#' Feedback and bug reports:
#' \url{https://github.com/Ecological-Complexity-Lab/MiRA/issues}.
#'
#' @return Invisibly returns the server handle. Use
#'   \code{httpuv::stopServer(handle)} to stop the server when done.
#'
#' @details
#' The function:
#' \enumerate{
#'   \item Converts the multilayer object to JSON via \code{multilayer_to_json}
#'   \item Starts a local HTTP server (using \code{httpuv}) that serves the
#'     MiRA app and the network JSON
#'   \item Opens the browser with auto-load enabled
#' }
#'
#' The server runs in the background. Call \code{httpuv::stopServer(handle)} or
#' close R to stop it. The JSON data is kept in memory and never written to disk.
#'
#' @seealso \code{multilayer_to_json, create_multilayer_network, load_emln}
#'
#' @export
#'
#' @examples
#' if (interactive()){
#' net <- load_emln(60)
#' srv <- plot_multilayer(net, bipartite = TRUE, setA_type = "pollinator")
#'
#' # When done:
#' httpuv::stopServer(srv)
#' }
plot_multilayer <- function(multilayer, bipartite = FALSE, setA_type = NULL,
                            directed = NULL, directed_interlayer = NULL,
                            port = 8080, mira_path = NULL,
                            browser = getOption("browser")) {
  message("Please report issues at ",
          "https://github.com/Ecological-Complexity-Lab/MiRA/issues")

  # Convert to JSON
  json_str <- multilayer_to_json(
    multilayer,
    bipartite           = bipartite,
    setA_type           = setA_type,
    directed            = directed,
    directed_interlayer = directed_interlayer
  )

  # Resolve MiRA path
  mira_path <- .resolve_mira_path(mira_path)
  message(sprintf("Serving MiRA from: %s", mira_path))

  # Create the httpuv app
  app <- list(
    call = function(req) {
      # Serve the network JSON at a special endpoint
      if (req$PATH_INFO == "/api/network.json") {
        return(list(
          status = 200L,
          headers = list(
            "Content-Type" = "application/json",
            "Access-Control-Allow-Origin" = "*",
            "Cache-Control" = "no-cache"
          ),
          body = json_str
        ))
      }

      # Serve static files from the MiRA directory
      # Map URL path to file system
      url_path <- req$PATH_INFO
      if (url_path == "/" || url_path == "") url_path <- "/index.html"

      file_path <- file.path(mira_path, gsub("^/", "", url_path))
      file_path <- normalizePath(file_path, mustWork = FALSE)

      # Security: ensure the path is within mira_path
      if (!startsWith(file_path, mira_path)) {
        return(list(status = 403L, headers = list(), body = "Forbidden"))
      }

      if (!file.exists(file_path) || dir.exists(file_path)) {
        return(list(status = 404L, headers = list(), body = "Not Found"))
      }

      # Determine content type
      ext <- tolower(tools::file_ext(file_path))
      content_types <- c(
        html = "text/html; charset=utf-8",
        htm = "text/html; charset=utf-8",
        js = "application/javascript; charset=utf-8",
        mjs = "application/javascript; charset=utf-8",
        css = "text/css; charset=utf-8",
        json = "application/json; charset=utf-8",
        png = "image/png",
        jpg = "image/jpeg",
        jpeg = "image/jpeg",
        svg = "image/svg+xml",
        gif = "image/gif",
        ico = "image/x-icon",
        woff = "font/woff",
        woff2 = "font/woff2"
      )
      ctype <- if (ext %in% names(content_types)) content_types[[ext]] else "application/octet-stream"

      # Read file
      if (ext %in% c("png", "jpg", "jpeg", "gif", "ico", "woff", "woff2")) {
        body <- readBin(file_path, "raw", file.info(file_path)$size)
      } else {
        body <- paste(readLines(file_path, warn = FALSE), collapse = "\n")
      }

      list(
        status = 200L,
        headers = list("Content-Type" = ctype, "Cache-Control" = "no-cache"),
        body = body
      )
    },
    onWSOpen = function(ws) {
      ws$close()
    } # No WebSocket support needed
  )

  # Start server on an available port
  max_attempts <- 50
  attempts <- 0
  server <- NULL

  while (is.null(server) && attempts < max_attempts) {
    tryCatch(
      {
        server <- httpuv::startServer("127.0.0.1", port, app)
      },
      error = function(e) {
        # Port in use, silently continue to the next one
      }
    )
    if (is.null(server)) {
      port <- port + 1
      attempts <- attempts + 1
    }
  }

  if (is.null(server)) {
    stop("Could not find an available port to start the server after ", max_attempts, " attempts.")
  }

  url <- sprintf("http://localhost:%d?autoload=true", port)
  message(sprintf("Server running at: %s", url))
  message("Call httpuv::stopServer(handle) to stop the server when done.")

  # Handle special case for Chrome on Mac
  if (!is.null(browser) && is.character(browser) &&
      tolower(browser) %in% c("chrome", "google chrome") &&
      Sys.info()["sysname"] == "Darwin") {
    browser <- "open -a 'Google Chrome'"
  }

  # Open browser
  utils::browseURL(url, browser = browser)

  invisible(server)
}


#' Resolve path to the bundled MiRA web app
#'
#' Looks first for an installed copy at
#' \code{system.file("MiRA", package = "emln")}, then falls back to
#' sibling/dev locations for in-source development.
#' @keywords internal
.resolve_mira_path <- function(mira_path = NULL) {
  if (!is.null(mira_path)) {
    if (!file.exists(file.path(mira_path, "index.html"))) {
      stop(sprintf(
        "`mira_path` does not contain index.html: %s", mira_path
      ))
    }
    return(normalizePath(mira_path, mustWork = TRUE))
  }

  # 1. Installed package: inst/MiRA
  installed <- system.file("MiRA", package = "emln")
  if (nzchar(installed) &&
      file.exists(file.path(installed, "index.html"))) {
    return(normalizePath(installed, mustWork = TRUE))
  }

  # 2. Dev fallbacks (running from emln source tree or a sibling checkout)
  cwd <- getwd()
  candidates <- c(
    file.path(cwd, "inst", "MiRA"),
    file.path(cwd, "..", "emln", "inst", "MiRA"),
    file.path(cwd, "..", "MiRA"),
    file.path(dirname(cwd), "MiRA")
  )
  for (cand in candidates) {
    if (file.exists(file.path(cand, "index.html"))) {
      return(normalizePath(cand, mustWork = TRUE))
    }
  }

  stop(
    "Could not find the MiRA web app.\n",
    "Looked in: system.file('MiRA', package = 'emln') and: ",
    paste(candidates, collapse = ", "), "\n",
    "If you are developing from source, pass `mira_path = ",
    "'path/to/MiRA'` explicitly, or ensure ",
    "inst/MiRA/index.html exists in your emln checkout."
  )
}

#'
#' @docType data
#' @title emln7_environment_rejmanek_stary_1979
#' 
#' @description Network Description:
#' 
#' | network_id | multilayer_network_type | mangal_code | ecological_network_type | state_nodes |
#' |:----------:|:-----------------------:|:-----------:|:-----------------------:|:-----------:|
#' |     7      |       Environment       |     112     |      Host-Parasite      |    FALSE    |
#' 
#' @format NULL
#' @usage NULL
#' @source mangal
#' @source https://mangal.io/doc/api/
#' @md
#' @keywords internal
'emln7_environment_rejmanek_stary_1979'

#'
#' @docType data
#' @title emln64_temporal_caradona_2020
#' 
#' @description Network Description:
#' 
#' | network_id | multilayer_network_type | ecological_network_type | state_nodes |
#' |:----------:|:-----------------------:|:-----------------------:|:-----------:|
#' |     64     |        Temporal         |       Pollination       |    FALSE    |
#' 
#' @format NULL
#' @usage NULL
#' @source EDI_data_portal
#' @source https://portal.edirepository.org/nis/mapbrowse?packageid=edi.512.1
#' @md
#' @keywords internal
'emln64_temporal_caradona_2020'

#'
#' @docType data
#' @title emln14_multiplex_mello_2019
#' 
#' @description Network Description:
#' 
#' | network_id | multilayer_network_type | ecological_network_type | state_nodes |
#' |:----------:|:-----------------------:|:-----------------------:|:-----------:|
#' |     14     |        Multiplex        |     Seed-Dispersal      |    FALSE    |
#' 
#' @format NULL
#' @usage NULL
#' @source Web_of_Life
#' @source http://www.web-of-life.es
#' @md
#' @keywords internal
'emln14_multiplex_mello_2019'

#'
#' @docType data
#' @title emln17_multiplex_traveset_2020
#'
#' @description Network Description:
#'
#' | network_id | multilayer_network_type | ecological_network_type | state_nodes |
#' |:----------:|:-----------------------:|:-----------------------:|:-----------:|
#' |     17     |        Multiplex        |        Multiple         |    FALSE    |
#'
#' @format NULL
#' @usage NULL
#' @source Figshare
#' @source https://doi.org/10.6084/m9.figshare.c.5208546.v1
#' @md
#' @keywords internal
'emln17_multiplex_traveset_2020'

#' Edge list to matrix conversion
#'
#' Convert an edge list to a matrix. Can handle unipartite and bipartite
#' networks.
#'
#' @details It is also possible to add node attributes. For this, it uses
#' igraph. The first column in the \code{node_metadata} data frame should have
#' the names of the nodes in the edge list.
#'
#' @param x An edge list of the format \code{from to weight}.
#' @param directed Is the network directed?
#' @param bipartite Is network bipartite?
#' @param group_names For bipartite networks: name of the groups in the columns
#'   and rows, respectively (e.g., parasites and hosts).
#' @param node_metadata Following the igraph method of \code{graph_from_data_frame}.
#'   Must have a column called node_name with names matching those in x.
#'
#' @return A \code{monolayer} object.
#' @seealso Functions \code{create_monolayer_network, monolayer} and \code{graph_from_data_frame} from igraph.
#'
#' @examples
#' library(dplyr)
#' network <- load_emln(17)
#' nodes <- network$nodes
#' links<-subset(network$extended_ids %>% filter(layer_from==1), select=c(node_from, node_to, weight))
#'
#' list_to_matrix(links, directed = TRUE, bipartite = FALSE, node_metadata = nodes)
#'
#'
#' # See examples in:
#' # https://emln.ecomplab.com/monolayer.html#Unipartite
#'
#' @export
#' @import dplyr
#' @importFrom igraph graph_from_data_frame V as_biadjacency_matrix as_adjacency_matrix

list_to_matrix <- function(x, directed=FALSE, bipartite=TRUE, group_names=c('set_cols','set_rows'), node_metadata=NULL){
  names(x)[3] <- 'weight'
  g <- igraph::graph_from_data_frame(x, directed = directed, vertices = node_metadata)
  if(bipartite){
    igraph::V(g)$type <- igraph::V(g)$name %in% as.data.frame(x)[,1] # As.data.frame is necessary because if x is a tibble then x[,1] does not work
    output_mat <- igraph::as_biadjacency_matrix(g, names = TRUE, attr = 'weight', sparse = FALSE)
  } else {
    output_mat <- igraph::as_adjacency_matrix(g, names = TRUE, sparse = FALSE, attr = 'weight')
    output_mat <- t(output_mat) # This is needed so from will be in columns and to in rows
  }

  # Create a table with node data
  nodes <- igraph::V(g)$name
  if (bipartite){
    node_list <- tibble::tibble(node_id=1:length(nodes),
                        node_group=c(rep(group_names[1],ncol(output_mat)),rep(group_names[2],nrow(output_mat))),
                        node_name=nodes)
  } else {
    node_list <- tibble::tibble(node_id=1:length(nodes),
                        node_name=nodes)
  }

  # message(dim(output_mat))
  if(any(rowSums(output_mat)==0)){message('Warning: One or more rows sum to 0. This may be ok if you expect some links with only outgoing links (e.g., basal species in a food web)')}
  if(any(colSums(output_mat)==0)){message('Warning: One or more columns sum to 0. This may be ok if you expect some links with only incoming links (e.g., top predators in a food web)')}

  out <- list(mode=ifelse(bipartite,'B','U'),
              directed=directed,
              nodes=node_list,
              mat=output_mat,
              edge_list=x,
              igraph_object=g)
  class(out) <- "monolayer"
  return(out)
}

el_unip <- data.frame(
  from   = c("A", "B", "C"),
  to     = c("B", "C", "A"),
  weight = c(1, 2, 3)
)

el_bip <- data.frame(
  from   = c("Plant1", "Plant1", "Plant2"),
  to     = c("Bee1", "Bee2", "Bee2"),
  weight = c(1, 2, 3)
)

test_that("list_to_matrix returns a monolayer for unipartite edge list", {
  result <- list_to_matrix(el_unip, directed = TRUE, bipartite = FALSE)
  expect_s3_class(result, "monolayer")
})

test_that("list_to_matrix sets mode U for unipartite", {
  result <- list_to_matrix(el_unip, directed = TRUE, bipartite = FALSE)
  expect_equal(result$mode, "U")
})

test_that("list_to_matrix returns a monolayer for bipartite edge list", {
  result <- list_to_matrix(el_bip, directed = FALSE, bipartite = TRUE)
  expect_s3_class(result, "monolayer")
})

test_that("list_to_matrix sets mode B for bipartite", {
  result <- list_to_matrix(el_bip, directed = FALSE, bipartite = TRUE)
  expect_equal(result$mode, "B")
})

test_that("list_to_matrix unipartite matrix is square", {
  result <- list_to_matrix(el_unip, directed = TRUE, bipartite = FALSE)
  expect_equal(nrow(result$mat), ncol(result$mat))
})

test_that("list_to_matrix bipartite matrix has correct dimensions", {
  result <- list_to_matrix(el_bip, directed = FALSE, bipartite = TRUE)
  n_from <- length(unique(el_bip$from))
  n_to   <- length(unique(el_bip$to))
  # incidence matrix: rows are one set, cols are the other
  expect_true(setequal(dim(result$mat), c(n_from, n_to)))
})

test_that("list_to_matrix has all required list elements", {
  result <- list_to_matrix(el_unip, directed = TRUE, bipartite = FALSE)
  expect_named(result, c("mode", "directed", "nodes", "mat", "edge_list", "igraph_object"))
})

test_that("list_to_matrix bipartite nodes table includes node_group", {
  result <- list_to_matrix(el_bip, directed = FALSE, bipartite = TRUE,
                            group_names = c("Plants", "Bees"))
  expect_true("node_group" %in% names(result$nodes))
  expect_setequal(unique(result$nodes$node_group), c("Plants", "Bees"))
})

test_that("list_to_matrix nodes table contains all unique nodes", {
  result <- list_to_matrix(el_unip, directed = TRUE, bipartite = FALSE)
  all_nodes <- unique(c(el_unip$from, el_unip$to))
  expect_setequal(result$nodes$node_name, all_nodes)
})

test_that("list_to_matrix igraph_object is an igraph", {
  result <- list_to_matrix(el_unip, directed = TRUE, bipartite = FALSE)
  expect_true(inherits(result$igraph_object, "igraph"))
})

#'
#' @docType data
#' @title emln43_spatial_montero_2005
#' 
#' @description Network Description:
#' 
#' | network_id | multilayer_network_type | ecological_network_type | state_nodes |
#' |:----------:|:-----------------------:|:-----------------------:|:-----------:|
#' |     43     |         Spatial         |       Pollination       |    TRUE     |
#' 
#' @format NULL
#' @usage NULL
#' @source Web_of_Life
#' @source http://www.web-of-life.es/map.php
#' @md
#' @keywords internal
'emln43_spatial_montero_2005'

#' Load networks include din the package
#'
#' @name load_emln
#'
#' @description Load a specific network by specifying its ID.
#'
#' @param network_id The network ID.
#'
#' @return A multilayer object (see ?multilayer).
#'
#' @details
#' Use the \code{search_emln} and \code{view_emln} functions to obtain network IDs.
#'
#' @examples
#' \donttest{
#' load_emln(network_id = 38)
#' }
#'
#' @seealso \code{multilayer}
#'
#' @export load_emln
#' @import dplyr
#' @import tidyr
#' @importFrom hablar retype
#' @importFrom stringr str_detect
#' @importFrom rlang .data

load_emln <- function(network_id) {
  #get the network name in order to load it -- depending on the network id entered
  #FOR DEBUG: network_name <- list.files('./data/',pattern = paste0('emln',as.character(network_id),'_'))
  pattern <- paste0("emln", network_id, "_")
  network_name <- descriptions %>% dplyr::filter(stringr::str_detect(.data$network_name, pattern)) %>% dplyr::slice(1) %>% dplyr::pull(.data$network_name)
  if (length(network_name) == 0) {
    return(paste0('Error: Network id',network_name, ' does not exist'))
  } else {
    #FOR DEBUG: network <- loadRData(paste0('./data/',network_name))
    network <- get(network_name)
  }

  #Pivot nodes to a wide format
  network$nodes %<>%
    tidyr::pivot_wider(names_from = 'attribute',values_from = 'value', values_fn = list) %>%
    tidyr::unnest(cols = everything())
    #fix the column types (so not all columns are character type)
    for (i in colnames(network$nodes)) {network$nodes[[i]] <- hablar::retype(network$nodes[[i]])}

  #if the same name id has different node id
  nodes_vector <- network$nodes[,c(1,2)] %>% dplyr::group_by(.data$node_id, .data$node_name) %>% dplyr::summarise()
  nodes_vector<- nodes_vector$node_name
  # nodes_vector <- network$nodes$node_name
  if(any(duplicated(nodes_vector))){
    message('WARNING! Some node names appear more than once:')
    message(nodes_vector[which(duplicated(nodes_vector))])
  }


  #Pivot layers to a wide format
  network$layers %<>%
    tidyr::pivot_wider(names_from = 'attribute', values_from = 'value', values_fn = list) %>%
    #if there are multiple types to the same layer, write them as individual rows and not lists
    tidyr::unnest(cols = everything()) %>%
    #remove type column
    dplyr::select(-"type") %>%
    dplyr::distinct() %>%
    # The layer_id and layer_name are necessary because they are used by functions like get_sam
    dplyr::rename(layer_id=layer) %>%
    dplyr::mutate(layer_name=paste('layer_',layer_id,sep=''))

  #fix the column types
  for (i in colnames(network$layers)) {
    if('date' %in% tolower(i)) {
      if(all(grepl(pattern = '/', x = network$layers$date))) { #if there are dates that are written with '/' (wrong format to fix)
        network$layers$date <- strptime(as.character(network$layers$date), "%d/%m/%Y") #fix the format
        format(network$layers$date, "%Y-%m-%d")
      }
    }
    network$layers[[i]] <- hablar::retype(network$layers[[i]])
  }

  if(any(duplicated(network$layers$layer_id))){
    message('WARNING! Some layer names appear more than once!')
  }

  #Pivot links to a wide format
  network$interactions %<>%
    tidyr::pivot_wider(names_from = 'attribute', values_from = 'value', values_fn = list) %>%
     # dplyr::select(interaction_id,layer_from,node_from,layer_to,node_to,weight) %>%
     tidyr::unnest(cols = everything()) %>%
     dplyr::select(-"interaction_id") %>%
    dplyr::select("layer_from", "node_from", "layer_to", "node_to", everything())

  #layer from and to are always ids
  #if the layer from and to are not in the layer ids, then convert to ids in edge list
  if (all(!(unique(network$interactions$layer_from) %in%  unique(network$layers$layer_id)))) {
  #if (!all(grepl(x = network$interactions$layer_from,pattern = '^[0-9]*$'))) {
  for (i in 1:nrow(network$interactions)) {
     network$interactions$layer_from[i] <- network$layers[network$layers$name == network$interactions$layer_from[i],]$layer_id
     }
}
  if (all(!(unique(network$interactions$layer_to) %in%  unique(network$layers$layer_id)))) {
  #if (!all(grepl(x = network$interactions$layer_to,pattern = '^[0-9]*$'))) {
    for (i in 1:nrow(network$interactions)) {
      network$interactions$layer_to[i] <- network$layers[network$layers$name == network$interactions$layer_to[i],]$layer_id
    }
  }

  #fix the column types
  for (i in colnames(network$interactions)) {network$interactions[[i]] <- hablar::retype(network$interactions[[i]])}

  # If the layers are ids, change them to layer_name
  if (all(grepl(x = network$interactions$layer_from,pattern = '^[0-9]*$'))) {network$interactions$layer_from <- paste('layer_',network$interactions$layer_from,sep='')}
  if (all(grepl(x = network$interactions$layer_to,pattern = '^[0-9]*$'))) {network$interactions$layer_to <- paste('layer_',network$interactions$layer_to,sep='')}

  # Create a map of state nodes
  message('Creating state node map')
  state_nodes_map <-
  dplyr::bind_rows(network$interactions %>% dplyr::distinct(layer_name=.data$layer_from, node_name=.data$node_from),
            network$interactions %>% dplyr::distinct(layer_name=.data$layer_to, node_name=.data$node_to)) %>%
    dplyr::distinct() %>%
    dplyr::left_join(network$nodes, by='node_name') %>%
    dplyr::left_join(network$layers %>% dplyr::select("layer_id", "layer_name"), by='layer_name') %>%
    dplyr::select("layer_id", "node_id", "layer_name", "node_name", everything())

  #Pivot state nodes to a wide format if more data on state nodes exist
  if ('state_nodes' %in% names(network)) {
  network$state_nodes %<>%
      tidyr::pivot_wider(names_from = 'attribute',values_from = 'value', values_fn = list) %>%
      tidyr::unnest(cols = everything())
  message('Joining with state node information in the data set')
  state_nodes_map %<>% dplyr::left_join(network$state_nodes %>% dplyr::select(-"node_name"), by=c('layer_id', 'node_id'))
  }

  ###  conversion of the node names to node ids in the edge list  ###
  message('Creating extended link list with node IDs')
  extended_edge_list <- extended_edge_list_ids <- as.data.frame(network$interactions)

  i=1
  for (i in 1:nrow(extended_edge_list)) {
    extended_edge_list_ids[i,2] <- unique(network$nodes[network$nodes$node_name == extended_edge_list[i,2],]$node_id)
    extended_edge_list_ids[i,4] <- unique(network$nodes[network$nodes$node_name == extended_edge_list[i,4],]$node_id)
    # Change layer names for ids
    extended_edge_list_ids[i,1] <- network$layers[network$layers$layer_name == extended_edge_list[i,1],]$layer_id
    extended_edge_list_ids[i,3] <- network$layers[network$layers$layer_name == extended_edge_list[i,3],]$layer_id
  }

  #fix the column type in extended_edge_list_ids
  for (i in colnames(extended_edge_list_ids)) {extended_edge_list_ids[[i]] <- hablar::retype(extended_edge_list_ids[[i]])}


  out <- list(nodes=network$nodes,
              layers=network$layers,
              extended=extended_edge_list,
              extended_ids=extended_edge_list_ids,
              state_nodes=state_nodes_map,
              description=network$description,
              references=network$references)
  class(out) <- 'multilayer'

  return(out)
}


# #check if there are networks that return an error when loading the emln
# errors_ids <- NULL
# for (network_id in 1:78){result <- tryCatch(expr = load_emln(network_id), error = function(e) {errors_ids<<-c(errors_ids,network_id)})}

test_that("load_emln returns a multilayer object", {
  result <- load_emln(1)
  expect_s3_class(result, "multilayer")
})

test_that("load_emln has all required element names", {
  result <- load_emln(1)
  expect_named(result, c("nodes", "layers", "extended", "extended_ids",
                          "state_nodes", "description", "references"))
})

test_that("load_emln nodes table has required columns", {
  result <- load_emln(1)
  expect_true(all(c("node_id", "node_name") %in% names(result$nodes)))
  expect_gt(nrow(result$nodes), 0)
})

test_that("load_emln layers table has required columns", {
  result <- load_emln(1)
  expect_true(all(c("layer_id", "layer_name") %in% names(result$layers)))
  expect_gt(nrow(result$layers), 0)
})

test_that("load_emln extended edge list has required columns", {
  result <- load_emln(1)
  expect_true(all(c("layer_from", "node_from", "layer_to", "node_to", "weight") %in%
                    names(result$extended)))
  expect_gt(nrow(result$extended), 0)
})

test_that("load_emln extended_ids uses numeric IDs for layers and nodes", {
  result <- load_emln(1)
  expect_true(is.numeric(result$extended_ids$layer_from))
  expect_true(is.numeric(result$extended_ids$node_from))
  expect_true(is.numeric(result$extended_ids$layer_to))
  expect_true(is.numeric(result$extended_ids$node_to))
})

test_that("load_emln state_nodes table has layer_id and node_id", {
  result <- load_emln(1)
  expect_true(all(c("layer_id", "node_id", "layer_name", "node_name") %in%
                    names(result$state_nodes)))
})

test_that("load_emln returns a character string for an invalid network_id", {
  result <- load_emln(9999)
  expect_true(is.character(result))
})

test_that("load_emln node IDs in nodes table are unique", {
  result <- load_emln(1)
  expect_equal(length(unique(result$nodes$node_id)), nrow(result$nodes))
})

test_that("load_emln layer IDs in layers table are unique", {
  result <- load_emln(1)
  expect_equal(length(unique(result$layers$layer_id)), nrow(result$layers))
})

#'
#' @docType data
#' @title emln54_spatial_snow b_k_&snow_1988
#'
#' @description Network Description:
#'
#' | network_id | multilayer_network_type | ecological_network_type | state_nodes |
#' |:----------:|:-----------------------:|:-----------------------:|:-----------:|
#' |     54     |         Spatial         |       Pollination       |    TRUE     |
#'
#' @format NULL
#' @usage NULL
#' @source Web_of_Life
#' @source Interaction_Web_DataBase
#' @source http://www.ecologia.ib.usp.br/iwdb/html/snow_snow_1971.htm
#' @source http://www.web-of-life.es/map.php?type=6
#' @md
#' @keywords internal
'emln54_spatial_snow_b_k_snow_1988'

#'
#' @docType data
#' @title emln48_environment_perturbation_bartomeus_2008
#' 
#' @description Network Description:
#' 
#' | network_id |  multilayer_network_type   | ecological_network_type | state_nodes |
#' |:----------:|:--------------------------:|:-----------------------:|:-----------:|
#' |     48     | Environment , Perturbation |       Pollination       |    FALSE    |
#' 
#' @format NULL
#' @usage NULL
#' @source Web_of_Life
#' @source Interaction_Web_DataBase
#' @source http://www.ecologia.ib.usp.br/iwdb/html/bartomeus_2008.html
#' @source http://www.web-of-life.es/map.php
#' @md
#' @keywords internal
'emln48_environment_perturbation_bartomeus_2008'

#'
#' @docType data
#' @title emln45_spatial_olesen_2002
#' 
#' @description Network Description:
#' 
#' | network_id | multilayer_network_type | mangal_code | ecological_network_type | state_nodes |
#' |:----------:|:-----------------------:|:-----------:|:-----------------------:|:-----------:|
#' |     45     |         Spatial         |     61      |       Pollination       |    FALSE    |
#' 
#' @format NULL
#' @usage NULL
#' @source mangal
#' @source https://mangal.io/doc/api/
#' @md
#' @keywords internal
'emln45_spatial_olesen_2002'

#'
#' @docType data
#' @title emln58_spatial_temporal_martins_2020
#' 
#' @description Network Description:
#' 
#' | network_id | multilayer_network_type | ecological_network_type | state_nodes |
#' |:----------:|:-----------------------:|:-----------------------:|:-----------:|
#' |     58     |   Spatial , Temporal    |     Plant-Herbivore     |    TRUE     |
#' 
#' @format NULL
#' @usage NULL
#' @source Dryad
#' @source https://doi.org/10.5061/dryad.vmcvdncq0
#' @md
#' @keywords internal
'emln58_spatial_temporal_martins_2020'

#'
#' @docType data
#' @title emln69_temporal_fang_2012
#' 
#' @description Network Description:
#' 
#' | network_id | multilayer_network_type | ecological_network_type | state_nodes |
#' |:----------:|:-----------------------:|:-----------------------:|:-----------:|
#' |     69     |        Temporal         |       Pollination       |    FALSE    |
#' 
#' @format NULL
#' @usage NULL
#' @source web
#' @source https://figshare.com/articles/dataset/Relative_Stability_of_Core_Groups_in_Pollination_Networks_in_a_Biodiversity_Hotspot_over_Four_Years/127797
#' @md
#' @keywords internal
'emln69_temporal_fang_2012'

#'
#' @docType data
#' @title emln25_spatial_carstensen_2014
#' 
#' @description Network Description:
#' 
#' | network_id | multilayer_network_type | ecological_network_type | state_nodes |
#' |:----------:|:-----------------------:|:-----------------------:|:-----------:|
#' |     25     |         Spatial         |       Pollination       |    FALSE    |
#' 
#' @format NULL
#' @usage NULL
#' @source figshare
#' @source https://figshare.com/articles/dataset/_Beta_Diversity_of_Plant_Pollinator_Networks_and_the_Spatial_Turnover_of_Pairwise_Interactions_/1234708
#' @md
#' @keywords internal
'emln25_spatial_carstensen_2014'

#' Export a multilayer network to JSON
#'
#' Converts an EMLN \code{multilayer} object to JSON format compatible with
#' MiRA, the multilayer interactive R-network app.
#'
#' @param multilayer A multilayer object (created by \code{create_multilayer_network}
#'   or \code{load_emln}).
#' @param file Optional file path to write the JSON to. If NULL, returns the JSON
#'   string.
#' @param bipartite Logical. Whether the network is bipartite. Defaults to
#'   \code{FALSE}. MiRA does not auto-detect bipartite structure — you must
#'   set this to \code{TRUE} explicitly. When \code{TRUE}, the nodes table
#'   must contain a \code{node_type} (or legacy \code{node_group}) column
#'   with exactly two distinct values across the network. Any \code{node_group}
#'   column is renamed to \code{node_type} in the output regardless of the
#'   bipartite flag (MiRA's canonical name).
#' @param setA_type Optional character. The \code{node_type} value to render
#'   as Set A (top row) in MiRA's bipartite layout. By ecological convention
#'   this is the higher trophic level (e.g. \code{"pollinator"},
#'   \code{"parasite"}, \code{"disperser"}). Only used when
#'   \code{bipartite = TRUE}. If \code{NULL} (default), MiRA falls back to
#'   alphabetical ordering of the two types.
#' @param directed Logical or NULL. Whether intralayer links are directed.
#'   If NULL (default), auto-detected by checking intralayer edge symmetry.
#' @param directed_interlayer Logical or NULL. Whether interlayer links are
#'   directed. If NULL (default), inherits from \code{directed} (matching
#'   MiRA's parser behavior). Set explicitly when intra- and interlayer
#'   directionality differ.
#'
#' @note
#' MiRA (Multilayer Interactive Rendering Application) is a browser-based
#' interactive visualization tool for multilayer networks, available at
#' \url{https://mira.ecomplab.com/}.
#'
#' If you use MiRA in published research, please cite:
#'
#' Nehoray SM, Bloch Y, Pilosof S (2026). Interactively visualizing biological
#' multilayer networks using MiRA. \emph{arXiv}:2605.09597 [cs.SI].
#' \doi{doi:10.48550/arXiv.2605.09597}
#'
#' Feedback and bug reports:
#' \url{https://github.com/Ecological-Complexity-Lab/MiRA/issues}.
#'
#' @return If \code{file} is NULL, returns the JSON string invisibly.
#'   If \code{file} is specified, writes JSON to disk and returns the file path
#'   invisibly.
#'
#' @details
#' The JSON output contains four arrays matching MiRA's expected format:
#' \itemize{
#'   \item \code{nodes}: Physical nodes with \code{node_id}, \code{node_name}, and
#'     any extra attributes. For bipartite networks, \code{node_group} is mapped to
#'     \code{node_type}.
#'   \item \code{layers}: Layer metadata with \code{layer_id}, \code{layer_name},
#'     and extra attributes. For bipartite networks, a \code{bipartite: true} flag
#'     is added; if \code{setA_type} is given it is also written per layer.
#'   \item \code{extended}: The extended edge list with \code{layer_from},
#'     \code{node_from}, \code{layer_to}, \code{node_to}, \code{weight}, and any
#'     link attributes.
#'   \item \code{state_nodes}: State node map with \code{layer_id}, \code{node_id},
#'     \code{layer_name}, \code{node_name}, plus any extra per-(layer, node)
#'     attributes (e.g. \code{abundance}, \code{module}).
#' }
#' Top-level flags \code{directed} and (when set) \code{directed_interlayer}
#' indicate edge directionality globally; they are not repeated per link.
#'
#' @seealso \code{plot_multilayer, create_multilayer_network, load_emln}
#'
#' @export
#' @import dplyr
#' @import tibble
#'
#' @examples
#' \donttest{
#' # Export to file
#' net <- load_emln(14)
#' multilayer_to_json(net, file = "tests/my_network.json", bipartite = TRUE)
#'
#' # Bipartite with explicit Set A ordering
#' multilayer_to_json(net, bipartite = TRUE, setA_type = "pollinator")
#'
#' # Get JSON string
#' json_str <- multilayer_to_json(net)
#' }

multilayer_to_json <- function(multilayer, file = NULL, bipartite = FALSE,
                               setA_type = NULL, directed = NULL,
                               directed_interlayer = NULL) {

  if (!inherits(multilayer, "multilayer")) {
    stop("Input must be a multilayer object (class 'multilayer').")
  }

  if (!is.logical(bipartite) || length(bipartite) != 1) {
    stop("`bipartite` must be a single TRUE/FALSE value (auto-detection has been removed).")
  }
  if (!is.null(setA_type)) {
    if (!is.character(setA_type) || length(setA_type) != 1 || is.na(setA_type)) {
      stop("`setA_type` must be a single character string or NULL.")
    }
    if (!isTRUE(bipartite)) {
      warning("`setA_type` is only used when bipartite = TRUE; ignoring.")
      setA_type <- NULL
    }
  }
  if (!is.null(directed_interlayer) &&
      (!is.logical(directed_interlayer) || length(directed_interlayer) != 1)) {
    stop("`directed_interlayer` must be a single TRUE/FALSE value or NULL.")
  }

  message("Please report issues at ",
          "https://github.com/Ecological-Complexity-Lab/MiRA/issues")

  # Resolve canonical type column. Precedence: node_type > node_group > type.
  # node_group and type are legacy names from older emln data.
  nodes_names <- names(multilayer$nodes)
  type_col <- if ("node_type" %in% nodes_names) "node_type"
              else if ("node_group" %in% nodes_names) "node_group"
              else if ("type" %in% nodes_names) "type"
              else NA_character_

  if (isTRUE(bipartite)) {
    if (is.na(type_col)) {
      stop("bipartite = TRUE but the nodes table has no `node_type` ",
           "(or legacy `node_group` / `type`) column.")
    }
    distinct_types <- unique(multilayer$nodes[[type_col]])
    distinct_types <- distinct_types[!is.na(distinct_types)]
    if (length(distinct_types) != 2) {
      stop(sprintf(
        paste0("bipartite = TRUE requires exactly 2 distinct values in `%s`",
               " (found %d: %s).\n",
               "If this network is not bipartite, use bipartite = FALSE."),
        type_col,
        length(distinct_types),
        paste(utils::head(distinct_types, 5), collapse = ", ")
      ))
    }
  }

  # ---- Auto-detect directed ----
  if (is.null(directed)) {
    # Check intralayer edge symmetry: if every (a->b) has a matching (b->a), it's undirected
    intra <- multilayer$extended %>%
      dplyr::filter(.data$layer_from == .data$layer_to)
    if (nrow(intra) > 0) {
      # Create a set of "layer|from|to" keys
      fwd_keys <- paste(intra$layer_from, intra$node_from, intra$node_to, sep = "|")
      rev_keys <- paste(intra$layer_from, intra$node_to, intra$node_from, sep = "|")
      directed <- !all(fwd_keys %in% rev_keys)
    } else {
      directed <- FALSE
    }
    message(sprintf("Auto-detected %s network.", ifelse(directed, "directed", "undirected")))
  }

  # ---- Build nodes array ----
  nodes_df <- as.data.frame(multilayer$nodes)
  # Canonicalize legacy names -> node_type (MiRA's canonical name).
  # Precedence: node_type > node_group > type.
  if (!("node_type" %in% names(nodes_df))) {
    if ("node_group" %in% names(nodes_df)) {
      names(nodes_df)[names(nodes_df) == "node_group"] <- "node_type"
    } else if ("type" %in% names(nodes_df)) {
      names(nodes_df)[names(nodes_df) == "type"] <- "node_type"
    }
  }
  # Dedupe by node_name — MiRA expects one physical node per row
  if ("node_name" %in% names(nodes_df)) {
    nodes_df <- nodes_df[!duplicated(nodes_df$node_name), , drop = FALSE]
  }
  # Ensure node_id column exists
  if (!("node_id" %in% names(nodes_df))) {
    nodes_df$node_id <- seq_len(nrow(nodes_df))
  }

  # ---- Build layers array ----
  layers_df <- as.data.frame(multilayer$layers)
  # Drop legacy 'name' column — layer_name is canonical
  layers_df <- layers_df[, names(layers_df) != "name", drop = FALSE]
  # Add bipartite flag and (optionally) setA_type per layer
  if (bipartite) {
    layers_df$bipartite <- TRUE
    if (!is.null(setA_type)) {
      # Validate setA_type matches a node_type present in the network
      type_values <- unique(nodes_df[["node_type"]])
      type_values <- type_values[!is.na(type_values)]
      if (!(setA_type %in% type_values)) {
        stop(sprintf(
          paste0("`setA_type = \"%s\"` does not match any node_type in the ",
                 "network (found: %s)."),
          setA_type, paste(type_values, collapse = ", ")
        ))
      }
      layers_df$setA_type <- setA_type
    }
  }

  # ---- Build extended edge list ----
  extended_df <- as.data.frame(multilayer$extended)
  # Ensure layer_from/layer_to use layer names (not IDs)
  # The extended edge list from create_multilayer_network already uses names.
  # From load_emln it also uses names (layer_1, layer_2, etc.)

  # Directed flag is set at the top level of the JSON (see below); no need
  # to tag each link individually.

  # ---- Build state_nodes array ----
  state_nodes_df <- as.data.frame(multilayer$state_nodes)
  # Backfill layer_name from layer_id if missing (older multilayer objects
  # sometimes store only the id)
  if (!("layer_name" %in% names(state_nodes_df)) &&
      "layer_id" %in% names(state_nodes_df) &&
      "layer_name" %in% names(layers_df)) {
    state_nodes_df$layer_name <- layers_df$layer_name[
      match(state_nodes_df$layer_id, layers_df$layer_id)
    ]
  }
  # Backfill node_id from node_name via the deduped nodes table
  if (!("node_id" %in% names(state_nodes_df)) &&
      "node_name" %in% names(state_nodes_df)) {
    state_nodes_df$node_id <- nodes_df$node_id[
      match(state_nodes_df$node_name, nodes_df$node_name)
    ]
  }
  # Backfill layer_id from layer_name
  if (!("layer_id" %in% names(state_nodes_df)) &&
      "layer_name" %in% names(state_nodes_df) &&
      "layer_id" %in% names(layers_df)) {
    state_nodes_df$layer_id <- layers_df$layer_id[
      match(state_nodes_df$layer_name, layers_df$layer_name)
    ]
  }
  # Keep core identity columns first, then any extra per-(layer, node)
  # attributes (e.g. abundance, module). MiRA's parser surfaces these in
  # the state-node attribute dropdown.
  core_cols <- c("layer_id", "node_id", "layer_name", "node_name")
  available_core <- intersect(core_cols, names(state_nodes_df))
  extra_cols <- setdiff(names(state_nodes_df), core_cols)
  state_nodes_df <- state_nodes_df[, c(available_core, extra_cols), drop = FALSE]

  # ---- Assemble JSON ----
  json_list <- list(directed = directed)
  if (!is.null(directed_interlayer)) {
    json_list$directed_interlayer <- directed_interlayer
  }
  json_list$nodes       <- .df_to_list_of_rows(nodes_df)
  json_list$layers      <- .df_to_list_of_rows(layers_df)
  json_list$extended    <- .df_to_list_of_rows(extended_df)
  json_list$state_nodes <- .df_to_list_of_rows(state_nodes_df)

  json_str <- .to_json(json_list)

  # ---- Output ----
  if (!is.null(file)) {
    writeLines(json_str, con = file)
    message(sprintf("JSON written to: %s", file))
    return(invisible(file))
  }

  return(invisible(json_str))
}


# ---- Internal helpers (no external JSON dependency) ----

#' Convert a data.frame to a list of named lists (one per row)
#' @keywords internal
.df_to_list_of_rows <- function(df) {
  lapply(seq_len(nrow(df)), function(i) {
    row <- as.list(df[i, , drop = FALSE])
    # Remove NA values to keep JSON clean
    row <- row[!sapply(row, function(x) is.na(x) || length(x) == 0)]
    row
  })
}

#' Minimal JSON serializer (no dependency on jsonlite)
#'
#' Converts an R list structure to a JSON string.
#' Supports: lists, vectors, strings, numbers, logicals, NULL.
#' @keywords internal
.to_json <- function(x, indent = 0) {
  pad <- function(n) paste(rep("  ", n), collapse = "")

  if (is.null(x)) {
    return("null")
  }

  if (is.logical(x) && length(x) == 1) {
    return(ifelse(x, "true", "false"))
  }

  if (is.numeric(x) && length(x) == 1) {
    # Use integer format when possible
    if (x == floor(x) && abs(x) < 1e15) {
      return(format(x, scientific = FALSE))
    }
    return(as.character(x))
  }

  if (is.character(x) && length(x) == 1) {
    # Escape special characters
    escaped <- gsub("\\\\", "\\\\\\\\", x)
    escaped <- gsub('"', '\\\\"', escaped)
    escaped <- gsub("\n", "\\\\n", escaped)
    escaped <- gsub("\r", "\\\\r", escaped)
    escaped <- gsub("\t", "\\\\t", escaped)
    return(paste0('"', escaped, '"'))
  }

  # Named list -> JSON object
  if (is.list(x) && !is.null(names(x))) {
    entries <- mapply(function(key, val) {
      paste0(pad(indent + 1), .to_json(key), ": ", .to_json(val, indent + 1))
    }, names(x), x, SIMPLIFY = FALSE, USE.NAMES = FALSE)
    return(paste0("{\n", paste(entries, collapse = ",\n"), "\n", pad(indent), "}"))
  }

  # Unnamed list -> JSON array
  if (is.list(x)) {
    entries <- lapply(x, function(item) {
      paste0(pad(indent + 1), .to_json(item, indent + 1))
    })
    return(paste0("[\n", paste(entries, collapse = ",\n"), "\n", pad(indent), "]"))
  }

  # Fallback: convert to string
  return(.to_json(as.character(x), indent))
}

#'
#' @docType data
#' @title emln51_spatial_primack_1983
#' 
#' @description Network Description:
#' 
#' | network_id | multilayer_network_type | ecological_network_type | state_nodes |
#' |:----------:|:-----------------------:|:-----------------------:|:-----------:|
#' |     51     |         Spatial         |       Pollination       |    TRUE     |
#' 
#' @format NULL
#' @usage NULL
#' @source Web_of_Life
#' @source http://www.web-of-life.es
#' @md
#' @keywords internal
'emln51_spatial_primack_1983'

#'
#' @docType data
#' @title emln73_temporal_ramos_robles_2016
#' 
#' @description Network Description:
#' 
#' | network_id | multilayer_network_type | ecological_network_type | state_nodes |
#' |:----------:|:-----------------------:|:-----------------------:|:-----------:|
#' |     73     |        Temporal         |     Seed-Dispersal      |    FALSE    |
#' 
#' @format NULL
#' @usage NULL
#' @source PeerJ
#' @source https://peerj.com/articles/2048/#supplemental-information
#' @md
#' @keywords internal
'emln73_temporal_ramos_robles_2016'

#' Convert a multilayer class to a supra-adjacency matrix
#'
#' This function creates a supra-adjacency matrix from a multilayer object.
#'
#' @param multilayer The multilayer object.
#' @param bipartite Is the network bipartite? Must be provided (no default
#'   value).
#' @param directed Is the network directed? Must be provided (no default value).
#' @param sparse Should the returned matrix be a sparse matrix (class Matrix)?
#'   Defaults to false.
#' @param remove_zero_rows_cols Should rows and columns that sum to 0 be
#'   removed? Defaults to false.
#'
#' @return A list:
#' \itemize{
#'   \item \code{M} The SAM. Can also be sparse.
#'   \item \code{nodes} A table of physical nodes, as in the multlayer object.
#'   \item \code{state_nodes_map} A table with state nodes to map the nodes to the row and columns of the SAM.
#' }
#'
#' @details By default, a SAM contains all the nodes in all the layers. However,
#'   not all nodes always occur in all layers. This will be reflected in the
#'   \code{state_nodes_map}: When layer and node ids are NA that means that the
#'   node did not occur in the layer.
#'
#'   A node may not have links across all
#'   the layers. In that case, its row or column sum will be zero. This can happen in a
#'   directed matrix (see toy example for unipartite network), or in
#'   diagonally-coupled networks (see toy bipartite example). The user can choose to remove rows and columns that sum to zero by setting \code{remove_zero_rows_cols} to \code{TRUE}.
#'
#' @examples
#'
#' # See examples in:
#' # https://emln.ecomplab.com/multilayer.html#To_supra-adjacency_matrices
#'
#' @export
#' @import dplyr
#' @importFrom Matrix Matrix
#' @importFrom rlang .data

get_sam <- function(multilayer, bipartite, directed, sparse=FALSE, remove_zero_rows_cols=FALSE) {
  # Create a map of state nodes
  nodes <- multilayer$nodes
  layer_attributes <- multilayer$layers
  state_nodes_map <- expand.grid(node_name=nodes$node_name, layer_name=layer_attributes$layer_name)
  state_nodes_map$sn_id <- 1:nrow(state_nodes_map)
  state_nodes_map$tuple <- paste(state_nodes_map$layer_name,state_nodes_map$node_name,sep='_')

  if (!bipartite) {
    # See https://github.com/manlius/muxViz/blob/master/gui-old/theory/README.md#operatively-rank-4-tensors-can-be-mapped-into-rank-2-supra-adjacency-matrices-to-facilitate-operations-with-care

      # Use state node names (the layer-node tuple) in the extended link list
    ell_tuples <- multilayer$extended %>%
      dplyr::mutate(from=paste(.data$layer_from, .data$node_from, sep='_')) %>%
      dplyr::mutate(to=paste(.data$layer_to, .data$node_to, sep='_')) %>%
      dplyr::select("from", "to", "weight")
    # The partial mat only contains the state nodes for which an interaction has been recorded
    partial_mat <- list_to_matrix(ell_tuples, bipartite = FALSE, directed = directed)$mat
    # Create the complete layer*nodes matrix
    M <- matrix(0, nrow = nrow(state_nodes_map), ncol = nrow(state_nodes_map), dimnames = list(state_nodes_map$tuple, state_nodes_map$tuple))
    # Embed the partial matrix inside the full one
    M[rownames(partial_mat), colnames(partial_mat)] <- partial_mat
   }

  if (bipartite){
    M <- matrix(0, nrow = nrow(state_nodes_map), ncol = nrow(state_nodes_map), dimnames = list(state_nodes_map$tuple, state_nodes_map$tuple))

    # Split the ell to layers
    intra <- multilayer$extended %>% dplyr::filter(.data$layer_from==.data$layer_to)
    # Need to convert to a factor to maintain the order of layers
    intra %<>% dplyr::mutate(layer_from = factor(.data$layer_from, levels = unique(.data$layer_from)))
    layers <- dplyr::group_split(intra, intra$layer_from)
    for (l in layers){
      layer_name <- l$layer_from[1]
      rect_matrix <- list_to_matrix(l[,c(2,4,5)], directed = FALSE, bipartite = TRUE)$mat
      # rect_matrix <- rect_matrix[sort(rownames(rect_matrix)),]
      # Get the dimensions of the rectangular matrix
      m <- nrow(rect_matrix)
      n <- ncol(rect_matrix)
      # Create a new square matrix with dimensions m+n
      rcnames <- c(colnames(rect_matrix),rownames(rect_matrix))
      rcnames <- paste(layer_name,rcnames,sep='_')
      square_matrix <- matrix(0, nrow = m + n, ncol = m + n, dimnames = list(rcnames, rcnames))
      # Copy the values from the rectangular matrix into the appropriate positions in the square matrix
      square_matrix[(n+1):(n+m), 1:n] <- rect_matrix
      square_matrix[1:n, (n+1):(n+m)] <- t(rect_matrix)

      # Populate the SAM
      M[rownames(square_matrix), colnames(square_matrix)] <- square_matrix
    }

    # Identify and populate interlayer interactions
    inter <- multilayer$extended %>% dplyr::filter(.data$layer_from!=.data$layer_to)
    # If there are interlayer links
    if (nrow(inter)>0){
      # Use state node names (the layer-node tuple) in the extended link list
      inter_tuples <- inter %>%
        dplyr::mutate(from=paste(.data$layer_from, .data$node_from, sep='_')) %>%
        dplyr::mutate(to=paste(.data$layer_to, .data$node_to, sep='_')) %>%
        dplyr::select("from", "to", "weight")
      # The interlayer matrix
      inter_mat <- list_to_matrix(inter_tuples, bipartite = FALSE, directed = directed)$mat
      # Populate the SAM with interlayer links
      M[rownames(inter_mat), colnames(inter_mat)] <- inter_mat
    }
  }

  # Some operations that are the same for the unipartite and bipartite

  # Change the row and column names to state node ids.
  colnames(M) <- state_nodes_map$sn_id[match(colnames(M),state_nodes_map$tuple)]
  rownames(M) <- state_nodes_map$sn_id[match(rownames(M),state_nodes_map$tuple)]
  # Create the full map. When layer and node ids are NA that means that the node did not occur in the layer
  state_nodes_map <- left_join(state_nodes_map, multilayer$state_nodes) %>%
    select("sn_id", "layer_name", "node_name", "layer_id", "node_id", "tuple")

  if (directed==FALSE & !isSymmetric(M)){warning('The SAM is not symmetric. Make sure this is what you expect because your network is not directed.')}

  if (remove_zero_rows_cols==TRUE){
    # Remove rows and columns whose sum is zero
    rows_to_keep <- rowSums(M) != 0
    cols_to_keep <- colSums(M) != 0
    M <- M[rows_to_keep, cols_to_keep]
  }

  # Make sparse?
  if (sparse){M <- Matrix::Matrix(M, sparse = TRUE)}

  out <- list(M=M, nodes=nodes, state_nodes_map=state_nodes_map)

  return(out)
}

# --- shared test fixtures ---

mat1 <- matrix(
  c(0, 1, 0,
    0, 0, 1,
    0, 0, 0),
  nrow = 3, byrow = TRUE,
  dimnames = list(c("A", "B", "C"), c("A", "B", "C"))
)
mat2 <- matrix(
  c(0, 0, 1,
    1, 0, 0,
    0, 1, 0),
  nrow = 3, byrow = TRUE,
  dimnames = list(c("A", "B", "C"), c("A", "B", "C"))
)

bip1 <- matrix(
  c(1, 0, 1,
    0, 1, 1),
  nrow = 2, byrow = TRUE,
  dimnames = list(c("Plant1", "Plant2"), c("Bee1", "Bee2", "Bee3"))
)
bip2 <- matrix(
  c(0, 1, 1,
    1, 0, 1),
  nrow = 2, byrow = TRUE,
  dimnames = list(c("Plant1", "Plant2"), c("Bee1", "Bee2", "Bee3"))
)

ml_unip <- create_multilayer_network(list(mat1, mat2),
                                      bipartite = FALSE, directed = TRUE)
ml_bip  <- create_multilayer_network(list(bip1, bip2),
                                      bipartite = TRUE,  directed = FALSE)

# --- tests ---

test_that("get_sam returns a list with M, nodes, state_nodes_map", {
  result <- get_sam(ml_unip, bipartite = FALSE, directed = TRUE)
  expect_named(result, c("M", "nodes", "state_nodes_map"))
})

test_that("get_sam M is a base matrix by default", {
  result <- get_sam(ml_unip, bipartite = FALSE, directed = TRUE)
  expect_true(is.matrix(result$M))
})

test_that("get_sam M dimensions equal nodes * layers for unipartite network", {
  result    <- get_sam(ml_unip, bipartite = FALSE, directed = TRUE)
  expected  <- nrow(ml_unip$nodes) * nrow(ml_unip$layers)
  expect_equal(nrow(result$M), expected)
  expect_equal(ncol(result$M), expected)
})

test_that("get_sam sparse=TRUE returns a sparseMatrix", {
  result <- get_sam(ml_unip, bipartite = FALSE, directed = TRUE, sparse = TRUE)
  expect_true(inherits(result$M, "sparseMatrix"))
})

test_that("get_sam remove_zero_rows_cols trims empty rows and cols", {
  result_full <- get_sam(ml_unip, bipartite = FALSE, directed = TRUE,
                          remove_zero_rows_cols = FALSE)
  result_trim <- get_sam(ml_unip, bipartite = FALSE, directed = TRUE,
                          remove_zero_rows_cols = TRUE)
  expect_true(nrow(result_trim$M) <= nrow(result_full$M))
  expect_true(ncol(result_trim$M) <= ncol(result_full$M))
})

test_that("get_sam state_nodes_map has required columns", {
  result <- get_sam(ml_unip, bipartite = FALSE, directed = TRUE)
  expect_true(all(c("sn_id", "layer_name", "node_name", "tuple") %in%
                    names(result$state_nodes_map)))
})

test_that("get_sam state_nodes_map row count equals nodes * layers", {
  result   <- get_sam(ml_unip, bipartite = FALSE, directed = TRUE)
  expected <- nrow(ml_unip$nodes) * nrow(ml_unip$layers)
  expect_equal(nrow(result$state_nodes_map), expected)
})

test_that("get_sam works for bipartite network", {
  result <- get_sam(ml_bip, bipartite = TRUE, directed = FALSE)
  expect_named(result, c("M", "nodes", "state_nodes_map"))
  expect_true(is.matrix(result$M))
})

test_that("get_sam bipartite M dimensions equal nodes * layers", {
  result   <- get_sam(ml_bip, bipartite = TRUE, directed = FALSE)
  expected <- nrow(ml_bip$nodes) * nrow(ml_bip$layers)
  expect_equal(nrow(result$M), expected)
  expect_equal(ncol(result$M), expected)
})

test_that("get_sam nodes in output match nodes in multilayer", {
  result <- get_sam(ml_unip, bipartite = FALSE, directed = TRUE)
  expect_equal(result$nodes, ml_unip$nodes)
})

#'
#' @docType data
#' @title emln16_multiplex_pocock_2012
#'
#' @description Network Description:
#'
#' | network_id | multilayer_network_type | ecological_network_type | state_nodes |
#' |:----------:|:-----------------------:|:-----------------------:|:-----------:|
#' |     16     |        Multiplex        |        Multiple         |    FALSE    |
#'
#' @format NULL
#' @usage NULL
#' @source Dryad
#' @source https://datadryad.org/stash/dataset/doi:10.5061/dryad.3s36r118
#' @md
#' @keywords internal
'emln16_multiplex_pocock_2012'

#' Convert a multilayer class to a a list of igraph objects
#'
#' This function creates a list of igraph objects from the network layers. Interlayer links are not included.
#'
#' @param multilayer The multilayer object.
#' @param bipartite Is the network bipartite? Must be provided (no default
#'   value).
#' @param directed Is the network directed? Must be provided (no default value).
#'
#' @return A list:
#' \itemize{
#'   \item \code{layers_igraph} A list of igraph objects, one per layer
#'   \item \code{nodes} A table of physical nodes, as in the multlayer object.
#'   \item \code{state_nodes_map} A table with state nodes to map the nodes to the row and columns of a SAM.
#' }
#'
#' @details If link, physical node or state node attributes are included in the multilayer network they are passed to the igraph objects.
#'
#' @examples
#'
#' # See examples in:
#' # https://emln.ecomplab.com/multilayer.html#To_igraph_objects
#'
#' @export
#' @importFrom igraph delete_vertex_attr
#' @import dplyr
#' @importFrom rlang .data

get_igraph <- function(multilayer, bipartite, directed) {
  # Create the SAM with all the state nodes
  nodes <- multilayer$nodes
  layer_attributes <- multilayer$layers
  state_nodes_map <- expand.grid(node_name=nodes$node_name, layer_name=layer_attributes$layer_name)
  state_nodes_map$sn_id <- 1:nrow(state_nodes_map)
  state_nodes_map$tuple <- paste(state_nodes_map$layer_name,state_nodes_map$node_name,sep='_')
  # Create the full map. When layer and node ids are NA that means that the node did not occur in the layer
  state_nodes_map <-
    dplyr::left_join(state_nodes_map, multilayer$state_nodes) %>%
    dplyr::select("sn_id", "layer_name", "node_name", "layer_id", "node_id", "tuple") %>%
    dplyr::left_join(nodes)
  # Split the ell to layers
  intra <- multilayer$extended %>% filter(.data$layer_from==.data$layer_to)
  # Need to convert to a factor to maintain the order of layers
  intra %<>% dplyr::mutate(layer_from = factor(.data$layer_from, levels = unique(.data$layer_from)))
  layers <- dplyr::group_split(intra, intra$layer_from)


  layers_igraph <- NULL
  # Work on each layer
  for (l in 1:length(layers)){
    lname <- layer_attributes$layer_name[l]
    net <- layers[[l]][,c(-1,-3)]
    if ("intra$layer_from" %in% names(net)) {net %<>% dplyr::select(-"intra$layer_from")} # this can be caused by the splitting
    # The next line was developed with the user-provided network examples
    nodes_in_layer <-
      multilayer$state_nodes %>%
      dplyr::filter(.data$layer_name==lname) %>%
      dplyr::select("node_name", everything())

    # Get the igraph object
    g <- list_to_matrix(x = net, directed = directed, bipartite = bipartite, node_metadata = nodes_in_layer)$igraph
    g <- igraph::delete_vertex_attr(g, "layer_name")

    layers_igraph[[l]] <- g
  }
  names(layers_igraph) <- layer_attributes$layer_name

  out <- list(layers_igraph=layers_igraph, nodes=nodes, state_nodes_map=state_nodes_map)

  return(out)
}

# --- shared test fixtures ---

mat1 <- matrix(
  c(0, 1, 0,
    0, 0, 1,
    0, 0, 0),
  nrow = 3, byrow = TRUE,
  dimnames = list(c("A", "B", "C"), c("A", "B", "C"))
)
mat2 <- matrix(
  c(0, 0, 1,
    1, 0, 0,
    0, 1, 0),
  nrow = 3, byrow = TRUE,
  dimnames = list(c("A", "B", "C"), c("A", "B", "C"))
)

bip1 <- matrix(
  c(1, 0, 1,
    0, 1, 1),
  nrow = 2, byrow = TRUE,
  dimnames = list(c("Plant1", "Plant2"), c("Bee1", "Bee2", "Bee3"))
)
bip2 <- matrix(
  c(0, 1, 1,
    1, 0, 1),
  nrow = 2, byrow = TRUE,
  dimnames = list(c("Plant1", "Plant2"), c("Bee1", "Bee2", "Bee3"))
)

ml_unip <- create_multilayer_network(list(mat1, mat2),
                                      bipartite = FALSE, directed = TRUE)
ml_bip  <- create_multilayer_network(list(bip1, bip2),
                                      bipartite = TRUE,  directed = FALSE)

# --- tests ---

test_that("get_igraph returns a list with layers_igraph, nodes, state_nodes_map", {
  result <- get_igraph(ml_unip, bipartite = FALSE, directed = TRUE)
  expect_named(result, c("layers_igraph", "nodes", "state_nodes_map"))
})

test_that("get_igraph returns one igraph object per layer", {
  result <- get_igraph(ml_unip, bipartite = FALSE, directed = TRUE)
  expect_equal(length(result$layers_igraph), nrow(ml_unip$layers))
})

test_that("get_igraph every element of layers_igraph is an igraph", {
  result <- get_igraph(ml_unip, bipartite = FALSE, directed = TRUE)
  expect_true(all(sapply(result$layers_igraph, inherits, what = "igraph")))
})

test_that("get_igraph layer names match multilayer layer names", {
  result <- get_igraph(ml_unip, bipartite = FALSE, directed = TRUE)
  expect_equal(names(result$layers_igraph), ml_unip$layers$layer_name)
})

test_that("get_igraph state_nodes_map has required columns", {
  result <- get_igraph(ml_unip, bipartite = FALSE, directed = TRUE)
  expect_true(all(c("sn_id", "layer_name", "node_name") %in%
                    names(result$state_nodes_map)))
})

test_that("get_igraph nodes in output match nodes in multilayer", {
  result <- get_igraph(ml_unip, bipartite = FALSE, directed = TRUE)
  expect_equal(result$nodes, ml_unip$nodes)
})

test_that("get_igraph works for bipartite network", {
  result <- get_igraph(ml_bip, bipartite = TRUE, directed = FALSE)
  expect_named(result, c("layers_igraph", "nodes", "state_nodes_map"))
  expect_equal(length(result$layers_igraph), nrow(ml_bip$layers))
  expect_true(all(sapply(result$layers_igraph, inherits, what = "igraph")))
})

#'
#' @docType data
#' @title emln23_spatial_canadian_fish_parasite
#' 
#' @description Network Description:
#' 
#' | network_id | multilayer_network_type | ecological_network_type | state_nodes |
#' |:----------:|:-----------------------:|:-----------------------:|:-----------:|
#' |     23     |         Spatial         |      Host-Parasite      |    TRUE     |
#' 
#' @format NULL
#' @usage NULL
#' @source Interaction_Web_DataBase
#' @source http://www.ecologia.ib.usp.br/iwdb/html/canadian_fish.html
#' @md
#' @keywords internal
'emln23_spatial_canadian_fish_parasite'

#'
#' @docType data
#' @title emln6_environment_kohler_2011
#' 
#' @description Network Description:
#' 
#' | network_id | multilayer_network_type | ecological_network_type | state_nodes |
#' |:----------:|:-----------------------:|:-----------------------:|:-----------:|
#' |     6      |       Environment       |       Pollination       |    TRUE     |
#' 
#' @format NULL
#' @usage NULL
#' @source Web_of_Life
#' @source Interaction_Web_DataBase
#' @source http://www.web-of-life.es/map.php?type=6
#' @source http://www.ecologia.ib.usp.br/iwdb/html/schleuning-et-al-2010.html
#' @md
#' @keywords internal
'emln6_environment_kohler_2011'

#'
#' @docType data
#' @title emln53_spatial_slovakia
#' 
#' @description Network Description:
#' 
#' | network_id | multilayer_network_type | ecological_network_type | state_nodes |
#' |:----------:|:-----------------------:|:-----------------------:|:-----------:|
#' |     53     |         Spatial         |      Host-Parasite      |    FALSE    |
#' 
#' @format NULL
#' @usage NULL
#' @source NA
#' @source NA
#' @md
#' @keywords internal
'emln53_spatial_slovakia'

#'
#' @docType data
#' @title emln68_temporal_environment_baeta_2011
#' 
#' @description Network Description:
#' 
#' | network_id | multilayer_network_type  | mangal_code | ecological_network_type | state_nodes |
#' |:----------:|:------------------------:|:-----------:|:-----------------------:|:-----------:|
#' |     68     | Temporal   , Environment |     73      |        Food-Web         |    FALSE    |
#' 
#' @format NULL
#' @usage NULL
#' @source mangal
#' @source https://mangal.io/doc/api/
#' @md
#' @keywords internal
'emln68_temporal_environment_baeta_2011'

#' Incidence matrix to edge list conversion
#'
#' Convert an incidence matrix to an edge list.
#'
#' @details Assigns column and row names if those do not exist.
#' Cannot handle node attributes/metadata. Always assumes an undirected network.
#'
#' @param x An incidence matrix.
#' @param group_names Name of the groups in the columns and rows, respectively (e.g., parasites and hosts).
#'
#' @return A \code{monolayer} object.
#' @seealso \code{create_monolayer_network, monolayer}
#'
#' @examples
#'
#' # See examples in: https://emln.ecomplab.com/monolayer.html#Bipartite
#'
#' matrix_to_list_bipartite(bipartite::memmott1999, group_names = c('Animals', 'Plants'))
#'
#' @export
#' @import dplyr
#' @importFrom igraph graph_from_biadjacency_matrix degree

matrix_to_list_bipartite <- function(x, group_names=c('set_cols','set_rows')){
  # Assign column and row names
  if (is.null(rownames(x))) {message('Assigning missing row names'); rownames(x) <- paste('R',1:nrow(x),sep='')}
  if (is.null(colnames(x))) {message('Assigning missing column names'); colnames(x) <- paste('C',1:ncol(x),sep='')}

  #Check that in the bipartite network no overlap between the sets of nodes.
  if (length(intersect(rownames(x),colnames(x)))>0)  {stop('ERROR: Check that there is no overlap between communities in the bipartite network.')}

  g <- igraph::graph_from_biadjacency_matrix(t(x), weighted = TRUE) # transposing ensures that "from" is at the columns and "to" is the rows
  l_bip <- as_tibble(igraph::as_data_frame(g, 'edges'))
  # summary(g)
  if(any(igraph::degree(g)==0)){message('Some node have no interactions. They will appear in the node table but not in the edge list')}
  nodes <- c((colnames(x)),(rownames(x)))
  node_list <- tibble(node_id=1:length(nodes),
                      node_group=c(rep(group_names[1],ncol(x)),rep(group_names[2],nrow(x))),
                      node_name=nodes)
  out <- list(mode='B', directed=FALSE, nodes=node_list, mat=x, edge_list=l_bip, igraph_object=g)
  class(out) <- "monolayer"
  return(out)
}

mat_bip <- matrix(
  c(1, 0, 1,
    0, 1, 1),
  nrow = 2, byrow = TRUE,
  dimnames = list(c("Plant1", "Plant2"), c("Bee1", "Bee2", "Bee3"))
)

mat_bip_nonames <- matrix(
  c(1, 0, 1,
    0, 1, 1),
  nrow = 2, byrow = TRUE
)

mat_bip_overlap <- matrix(
  c(1, 0, 1,
    0, 1, 1),
  nrow = 2, byrow = TRUE,
  dimnames = list(c("A", "B"), c("A", "C", "D"))  # "A" overlaps
)

test_that("matrix_to_list_bipartite returns a monolayer object", {
  result <- matrix_to_list_bipartite(mat_bip, group_names = c("Bees", "Plants"))
  expect_s3_class(result, "monolayer")
})

test_that("matrix_to_list_bipartite sets mode to B and is undirected", {
  result <- matrix_to_list_bipartite(mat_bip)
  expect_equal(result$mode, "B")
  expect_false(result$directed)
})

test_that("matrix_to_list_bipartite has all required list elements", {
  result <- matrix_to_list_bipartite(mat_bip)
  expect_named(result, c("mode", "directed", "nodes", "mat", "edge_list", "igraph_object"))
})

test_that("matrix_to_list_bipartite nodes table has node_group column", {
  result <- matrix_to_list_bipartite(mat_bip, group_names = c("Bees", "Plants"))
  expect_true("node_group" %in% names(result$nodes))
})

test_that("matrix_to_list_bipartite nodes table has correct total count", {
  result <- matrix_to_list_bipartite(mat_bip)
  # 2 row nodes + 3 col nodes
  expect_equal(nrow(result$nodes), 5)
})

test_that("matrix_to_list_bipartite assigns custom group names correctly", {
  result <- matrix_to_list_bipartite(mat_bip, group_names = c("Bees", "Plants"))
  expect_setequal(unique(result$nodes$node_group), c("Bees", "Plants"))
})

test_that("matrix_to_list_bipartite edge list has from, to, weight columns", {
  result <- matrix_to_list_bipartite(mat_bip)
  expect_true(all(c("from", "to", "weight") %in% names(result$edge_list)))
})

test_that("matrix_to_list_bipartite errors when row and col names overlap", {
  expect_error(matrix_to_list_bipartite(mat_bip_overlap))
})

test_that("matrix_to_list_bipartite assigns names when missing", {
  result <- matrix_to_list_bipartite(mat_bip_nonames)
  expect_s3_class(result, "monolayer")
  expect_equal(nrow(result$nodes), 5)
})

test_that("matrix_to_list_bipartite igraph_object is an igraph", {
  result <- matrix_to_list_bipartite(mat_bip)
  expect_true(inherits(result$igraph_object, "igraph"))
})

#' @keywords internal
"_PACKAGE"

## usethis namespace: start
## usethis namespace: end

utils::globalVariables(c("descriptions",
                         "from", "to", "layer", "layer_id", "name"))

#'
#' @docType data
#' @title emln77_temporal_vaiela_1974
#' 
#' @description Network Description:
#' 
#' | network_id | multilayer_network_type | mangal_code | ecological_network_type | state_nodes |
#' |:----------:|:-----------------------:|:-----------:|:-----------------------:|:-----------:|
#' |     77     |        Temporal         |     123     |        Food-Web         |    FALSE    |
#' 
#' @format NULL
#' @usage NULL
#' @source mangal
#' @source https://mangal.io/doc/api/
#' @md
#' @keywords internal
'emln77_temporal_vaiela_1974'

#'
#' @docType data
#' @title emln29_spatial_dupont_olesen_2009
#' 
#' @description Network Description:
#' 
#' | network_id | multilayer_network_type | ecological_network_type | state_nodes |
#' |:----------:|:-----------------------:|:-----------------------:|:-----------:|
#' |     29     |         Spatial         |       Pollination       |    FALSE    |
#' 
#' @format NULL
#' @usage NULL
#' @source Web_of_Life
#' @source http://www.web-of-life.es/map.php
#' @md
#' @keywords internal
'emln29_spatial_dupont_olesen_2009'

#'
#' @docType data
#' @title emln49_spatial_perturbation_vazquez_2002
#' 
#' @description Network Description:
#' 
#' | network_id |  multilayer_network_type   | ecological_network_type | state_nodes |
#' |:----------:|:--------------------------:|:-----------------------:|:-----------:|
#' |     49     | Spatial     , Perturbation |       Pollination       |    FALSE    |
#' 
#' @format NULL
#' @usage NULL
#' @source Interaction_Web_DataBase
#' @source http://www.ecologia.ib.usp.br/iwdb/html/vazquez_2002.html
#' @md
#' @keywords internal
'emln49_spatial_perturbation_vazquez_2002'

#'
#' @docType data
#' @title emln13_multiplex_kefi_2016
#' 
#' @description Network Description:
#' 
#' | network_id | multilayer_network_type | ecological_network_type | state_nodes |
#' |:----------:|:-----------------------:|:-----------------------:|:-----------:|
#' |     13     |        Multiplex        |        Food-Web         |    FALSE    |
#' 
#' @format NULL
#' @usage NULL
#' @source Dryad
#' @source https://datadryad.org/stash/dataset/doi:10.5061/dryad.b4vg0
#' @md
#' @keywords internal
'emln13_multiplex_kefi_2016'

#' View networks
#'
#' @name view_emln
#'
#' @description View the database interactively.
#'
#' @return An interactive GUI for browinsg through networks.
#'
#' @export view_emln
#' @import DT



view_emln <- function() {
  #load('./data/descriptions.rda')
  if (!requireNamespace("DT", quietly = TRUE)) {
    stop(
      "Package \"DT\" must be installed to use this function.",
      call. = FALSE
    )
  }
  datatable(descriptions,rownames = FALSE)
}

#'
#' @docType data
#' @title emln65_temporal_chacoff_2020
#' 
#' @description Network Description:
#' 
#' | network_id | multilayer_network_type | ecological_network_type | state_nodes |
#' |:----------:|:-----------------------:|:-----------------------:|:-----------:|
#' |     65     |        Temporal         |       Pollination       |    FALSE    |
#' 
#' @format NULL
#' @usage NULL
#' @source Dryad
#' @source https://datadryad.org/stash/dataset/doi:10.5061/dryad.8cz8w9gm1
#' @md
#' @keywords internal
'emln65_temporal_chacoff_2020'

#' Export a multilayer network to CSV files
#'
#' Writes an EMLN \code{multilayer} object as a set of CSV files consumed
#' by MiRA's CSV importer. Three files are always written; a fourth
#' (\code{<prefix>_state_nodes.csv}) is added when per-(layer, node)
#' attributes are present.
#'
#' @param multilayer A multilayer object (created by
#'   \code{create_multilayer_network} or \code{load_emln}).
#' @param dir Directory to write the CSV files to. Created if it does not
#'   already exist.
#' @param prefix Character. File name prefix. Defaults to \code{"network"}.
#'   Three files are always written:
#'   \code{<prefix>_edges.csv}, \code{<prefix>_layers.csv},
#'   \code{<prefix>_nodes.csv}.
#'   A fourth file \code{<prefix>_state_nodes.csv} is written when the
#'   multilayer object contains per-(layer, node) attributes beyond
#'   \code{layer_name} and \code{node_name} (e.g. abundance,
#'   module).
#' @param bipartite Logical. Whether the network is bipartite. Defaults to
#'   \code{FALSE}. When \code{TRUE}, the nodes table must contain a
#'   \code{node_type} (or legacy \code{node_group}) column with exactly two
#'   distinct values across the network, and a \code{bipartite} column is
#'   written to the layers CSV.
#' @param setA_type Optional character. The \code{node_type} value to render
#'   as Set A (top row) in MiRA's bipartite layout. By ecological convention
#'   this is the higher trophic level (e.g. \code{"pollinator"},
#'   \code{"parasite"}, \code{"disperser"}). Only used when
#'   \code{bipartite = TRUE}; written as a \code{setA_type} column in the
#'   layers CSV. If \code{NULL} (default), MiRA falls back to alphabetical
#'   ordering of the two types.
#' @param directed Logical or NULL. Whether the network is directed. If NULL
#'   (default), auto-detected by checking intralayer edge symmetry.
#'
#' @note
#' MiRA (Multilayer Interactive Rendering Application) is a browser-based
#' interactive visualization tool for multilayer networks, available at
#' \url{https://mira.ecomplab.com/}.
#'
#' If you use MiRA in published research, please cite:
#'
#' Nehoray SM, Bloch Y, Pilosof S (2026). Interactively visualizing biological
#' multilayer networks using MiRA. \emph{arXiv}:2605.09597 [cs.SI].
#' \doi{doi:10.48550/arXiv.2605.09597}
#'
#' Feedback and bug reports:
#' \url{https://github.com/Ecological-Complexity-Lab/MiRA/issues}.
#'
#' @return Invisibly returns a named character vector of file paths written
#'   (\code{edges}, \code{layers}, \code{nodes}, and optionally
#'   \code{state_nodes}).
#'
#' @details
#' The CSV files follow the schema accepted by MiRA's CSV importer:
#' \itemize{
#'   \item \strong{edges}: \code{layer_from, node_from, layer_to, node_to,
#'     weight} (+ any extra link attributes).
#'   \item \strong{layers}: \code{layer_id, layer_name} (+ \code{latitude,
#'     longitude, bipartite} and any extra layer attributes).
#'   \item \strong{nodes}: \code{node_name} (+ \code{node_type} and any extra
#'     physical node attributes, same across all layers).
#'     \code{node_group} is renamed to \code{node_type}.
#'   \item \strong{state_nodes} (written only when extra per-(layer, node)
#'     attributes exist): \code{layer_name, node_name} + extra columns.
#'     Use this when the same node has different attribute values in
#'     different layers (e.g. abundance, module membership).
#' }
#' The \code{directed} flag is not written to the CSV files; it is a property
#' you specify in MiRA's CSV import dialog at load time.
#'
#' @seealso \code{\link{multilayer_to_json}}, \code{\link{plot_multilayer}}
#'
#' @export
#'
#' @examples
#' \donttest{
#' net <- load_emln(14)
#' multilayer_to_csv(net, dir = "tests/out/", prefix = "kefi", bipartite = TRUE)
#' }
multilayer_to_csv <- function(multilayer, dir, prefix = "network",
                              bipartite = FALSE, setA_type = NULL,
                              directed = NULL) {

  if (!inherits(multilayer, "multilayer")) {
    stop("Input must be a multilayer object (class 'multilayer').")
  }
  if (!is.logical(bipartite) || length(bipartite) != 1) {
    stop("`bipartite` must be a single TRUE/FALSE value.")
  }
  if (!is.character(dir) || length(dir) != 1) {
    stop("`dir` must be a single directory path.")
  }
  if (!is.null(setA_type)) {
    if (!is.character(setA_type) || length(setA_type) != 1 || is.na(setA_type)) {
      stop("`setA_type` must be a single character string or NULL.")
    }
    if (!isTRUE(bipartite)) {
      warning("`setA_type` is only used when bipartite = TRUE; ignoring.")
      setA_type <- NULL
    }
  }

  message("Please report issues at ",
          "https://github.com/Ecological-Complexity-Lab/MiRA/issues")

  if (!dir.exists(dir)) {
    dir.create(dir, recursive = TRUE)
  }

  # ---- Nodes ----
  nodes_df <- as.data.frame(multilayer$nodes)
  # Canonicalize legacy names -> node_type.
  # Precedence: node_type > node_group > type.
  if (!("node_type" %in% names(nodes_df))) {
    if ("node_group" %in% names(nodes_df)) {
      names(nodes_df)[names(nodes_df) == "node_group"] <- "node_type"
    } else if ("type" %in% names(nodes_df)) {
      names(nodes_df)[names(nodes_df) == "type"] <- "node_type"
    }
  }
  # Dedupe by node_name
  if ("node_name" %in% names(nodes_df)) {
    nodes_df <- nodes_df[!duplicated(nodes_df$node_name), , drop = FALSE]
  }

  # Bipartite validation (match multilayer_to_json semantics)
  if (isTRUE(bipartite)) {
    if (!("node_type" %in% names(nodes_df))) {
      stop("bipartite = TRUE but the nodes table has no `node_type` ",
           "(or legacy `node_group`) column.")
    }
    distinct_types <- unique(nodes_df$node_type)
    distinct_types <- distinct_types[!is.na(distinct_types)]
    if (length(distinct_types) != 2) {
      stop(sprintf(
        "bipartite = TRUE requires exactly 2 distinct values in `node_type` (found %d).",
        length(distinct_types)
      ))
    }
  }

  # ---- Layers ----
  layers_df <- as.data.frame(multilayer$layers)
  layers_df <- layers_df[, names(layers_df) != "name", drop = FALSE]
  if (isTRUE(bipartite)) {
    layers_df$bipartite <- TRUE
    if (!is.null(setA_type)) {
      type_values <- unique(nodes_df$node_type)
      type_values <- type_values[!is.na(type_values)]
      if (!(setA_type %in% type_values)) {
        stop(sprintf(
          paste0("`setA_type = \"%s\"` does not match any node_type in the ",
                 "network (found: %s)."),
          setA_type, paste(type_values, collapse = ", ")
        ))
      }
      layers_df$setA_type <- setA_type
    }
  }

  # ---- Edges (extended list) ----
  edges_df <- as.data.frame(multilayer$extended)

  # ---- State nodes (optional) ----
  # Write a state_nodes CSV when the multilayer object carries per-(layer, node)
  # attributes beyond the identity columns (layer_name, node_name, layer_id,
  # node_id).  These differ from physical node attributes because the same node
  # can have different values in different layers (e.g. abundance, module).
  state_nodes_path <- NULL
  state_nodes_df   <- NULL
  if (!is.null(multilayer$state_nodes)) {
    sn <- as.data.frame(multilayer$state_nodes)
    core_cols <- c("layer_id", "node_id", "layer_name", "node_name")
    extra_cols <- setdiff(names(sn), core_cols)
    if (length(extra_cols) > 0) {
      keep <- intersect(c("layer_name", "node_name", extra_cols), names(sn))
      state_nodes_df   <- sn[, keep, drop = FALSE]
      state_nodes_path <- file.path(dir, paste0(prefix, "_state_nodes.csv"))
    }
  }

  # ---- Write files ----
  edges_path  <- file.path(dir, paste0(prefix, "_edges.csv"))
  layers_path <- file.path(dir, paste0(prefix, "_layers.csv"))
  nodes_path  <- file.path(dir, paste0(prefix, "_nodes.csv"))

  utils::write.csv(edges_df,  edges_path,  row.names = FALSE, na = "")
  utils::write.csv(layers_df, layers_path, row.names = FALSE, na = "")
  utils::write.csv(nodes_df,  nodes_path,  row.names = FALSE, na = "")

  paths <- c(edges = edges_path, layers = layers_path, nodes = nodes_path)

  if (!is.null(state_nodes_path)) {
    utils::write.csv(
      state_nodes_df, state_nodes_path, row.names = FALSE, na = ""
    )
    paths <- c(paths, state_nodes = state_nodes_path)
    message(sprintf("Wrote:\n  %s\n  %s\n  %s\n  %s",
                    edges_path, layers_path, nodes_path, state_nodes_path))
  } else {
    message(sprintf("Wrote:\n  %s\n  %s\n  %s",
                    edges_path, layers_path, nodes_path))
  }

  if (!is.null(directed)) {
    message(sprintf(
      "Note: directed=%s is not stored in the CSV files; set it in ",
      directed
    ), "MiRA's CSV import dialog at load time.")
  }

  invisible(paths)
}

#'
#' @docType data
#' @title emln41_spatial_melian_2015
#' 
#' @description Network Description:
#' 
#' | network_id | multilayer_network_type | ecological_network_type | state_nodes |
#' |:----------:|:-----------------------:|:-----------------------:|:-----------:|
#' |     41     |         Spatial         |        Food-Web         |    FALSE    |
#' 
#' @format NULL
#' @usage NULL
#' @source Dryad
#' @source https://datadryad.org/stash/dataset/doi:10.5061/dryad.06bb5
#' @md
#' @keywords internal
'emln41_spatial_melian_2015'

#'
#' @docType data
#' @title emln1_environment_schleuning_2010
#' 
#' @description Network Description:
#' 
#' | network_id | multilayer_network_type | ecological_network_type | state_nodes |
#' |:----------:|:-----------------------:|:-----------------------:|:-----------:|
#' |     1      |       Environment       |     Seed-Dispersal      |    FALSE    |
#' 
#' @format NULL
#' @usage NULL
#' @source Interaction_Web_DataBase
#' @source http://www.ecologia.ib.usp.br/iwdb/html/schleuning-et-al-2010.html
#' @md
#' @keywords internal
'emln1_environment_schleuning_2010'

#' Internal function - loads an RData file, and returns it
#'
#' @param fileName file to be downloaded
#' @keywords internal
#' @import tibble

loadRData <- function(fileName){
load(fileName)
get(ls()[ls() != "fileName"])
}

#' Create Multilayer Network
#'
#' This function creates a multilayer network from multiple matrices / link
#' lists.
#'
#' @param list_of_layers List of matrices or list of link lists (format
#'   \code{from to weight}; see description). These are the intralayer links.
#' @param bipartite Is the network bipartite? Must be provided (no default
#'   value).
#' @param directed Is the network directed? Must be provided (no default value).
#' @param interlayer_links Interlayer extended edge list of the format
#'   \code{layer_from node_from layer_to node_to weight}. Default is NULL,
#'   assuming no interlayer links.
#' @param layer_attributes Optional. A data frame with layer attributes. The
#'   first column must be called \code{layer_id}, and it must have unique values (one row per layer).
#'   Layer name column, if exists must be called \code{layer_name}, and its values must be unique. The
#'   order of the rows and layer_id should be the same as in \code{list_of_layers}.
#'   No column is allowed to be named \code{name}.
#' @param state_node_attributes Optional. Additional information on physical nodes in
#'   layers. Must contain columns \code{layer_name} and \code{node_name}.
#' @param physical_node_attributes Optional. Additional information on physical nodes. Must contain column \code{node_name}.
#'
#' @return A multilayer object (see ?multilayer).
#'
#' @details Input of \code{list_of_layers} is handled by the function
#'   \code{create_monolayer_network}; see it's description for details on input.
#'   If using matrices as input, they should contain row and column names. If link lists are provided as input column names should be
#'   \code{from to weight}.
#'
#'   When using link lists, it is possible to include link
#'   attributes. In that case the headers of all the link lists and, if
#'   included, the interlayer links, must be the same (after the weight), even if some attributes
#'   are included in only some layers (see detailed example in the code accompanying the package). Missing link attributes can be set to \code{NA}.
#'
#'   If interlayer links are provided, then layer_attributes must also be
#'   provided. Specify layer and nodes by UNIQUE names (not IDs). Layer names should
#'   correspond to those provided in layer_attributes.
#'
#'   Attributes of state nodes can be provided with \code{state_node_attributes}. If
#'   provided, then columns \code{layer_name} and \code{node_name} must be
#'   included.
#'
#'   For compatibility with function \code{load_emln} the output contains
#'   \code{description} and \code{references}. The value of any missing data
#'   frames in the multilayer object will be set to NULL.
#'
#'   See multiple example in the code accompanying the package
#'
#' @seealso \code{multilayer, create_monolayer_network}
#'
#' @export
#' @importFrom rlang .data
#'
#' @examples
#' library(tibble)
#' pond_1 <- matrix(c(0,1,1,0,0,1,0,0,0), byrow = TRUE,
#' nrow = 3, ncol = 3, dimnames = list(c('pelican','fish','crab'),
#' c('pelican','fish','crab')))
#' pond_2 <- matrix(c(0,1,0,0,0,1,0,0,0), byrow = TRUE,
#' nrow = 3, ncol = 3, dimnames = list(c('pelican','fish','crab'),
#' c('pelican','fish','crab')))
#' pond_3 <- matrix(c(0,1,1,0,0,1,0,0,0), byrow = TRUE,
#' nrow = 3, ncol = 3, dimnames = list(c('pelican','fish','tadpole'),
#' c('pelican','fish','tadpole')))
#'
#' layer_attrib <- tibble(layer_id=1:3,
#'                        layer_name=c('pond_1','pond_2','pond_3'),
#'                        location=c('valley','mid-elevation','uphill'))
#'
#' # Create the ELL tibble with interlayer links.
#' interlayer <- tibble(layer_from=c('pond_1','pond_1','pond_1'),
#'                      node_from=c('pelican','crab','pelican'),
#'                      layer_to=c('pond_2','pond_2','pond_3'),
#'                     node_to=c('pelican','crab','pelican'),
#'                      weight=1)
#'
#' # This is a directed network so the links should go both ways,
#' # even though they are symmetric.
#' interlayer_2 <- interlayer[,c(3,4,1,2,5)]
#' names(interlayer_2) <- names(interlayer)
#' interlayer <- rbind(interlayer, interlayer_2)
#'
#' multilayer <- create_multilayer_network(list_of_layers =
#' list(pond_1, pond_2, pond_3), layer_attributes = layer_attrib,
#' interlayer_links = interlayer, bipartite = FALSE, directed = TRUE)
#'

create_multilayer_network <- function(list_of_layers, bipartite, directed, interlayer_links = NULL, layer_attributes = NULL, state_node_attributes = NULL, physical_node_attributes = NULL) {

  # When link lists are used then it is possible to add link attributes. Check if all the headers are the same for all layers.
  if ("data.frame" %in% class(list_of_layers[[1]])){
    headers <- lapply(list_of_layers, names)
    all_identical <- all(sapply(headers, function(x) identical(x, headers[[1]])))
    if (all_identical==FALSE){stop('The headers of all the link lists in list_of_layers must be identical')}

    # To compare with interlayer links take only the headers after the weight
    if (!is.null(interlayer_links)){
      intralayer_headers <- names(list_of_layers[[1]])
      intralayer_headers <- intralayer_headers[4:length(intralayer_headers)]
      interlayer_headers <- names(interlayer_links)[6:length(interlayer_links)]
      if(!identical(interlayer_headers, intralayer_headers)){stop('The link attributes of intralayer and interlayer links must be identical')}
    }
  }

  # Initialize
  extended_edge_list <- NULL
  state_nodes <- NULL

  # Check validity of layer attribute table
  if (is.null(layer_attributes)) {
    message('Layer attributes not provided, I added them (see layer_attributes in the final object)')
    layer_attributes <- data.frame(layer_id = 1:length(list_of_layers), layer_name=paste('layer_',1:length(list_of_layers),sep = ''))
  } else {
    layer_attributes <- data.frame(layer_attributes) # working with data frame is easier than with tibbles.
    if (names(layer_attributes)[1]!='layer_id') {stop('First column in layer attributes should be layer_id, and the order should be as provied in the list of layers.')}
    if (nrow(layer_attributes)!=length(list_of_layers)) {stop('The number of layers between list_of_layers and layer_attributes should be consistent.')}

    # check if layer_name exists, if not then create it.
    if (!'layer_name' %in% names(layer_attributes)) {
      message('Layer name column not provided, will be generated')
      layer_attributes$layer_name <- paste('layer_',1:length(list_of_layers),sep = '')}
    else {# make sure the values are unique.
      if (any(duplicated(layer_attributes$layer_name))==TRUE){stop('Layer names should be unique.')}
    }

    # enforce not using the name 'name' for any column in the layer attributes
    if ('name' %in% names(layer_attributes)){stop('The column name "name" is not allowed for any column in the layer attributes. Please change the name of the column.')}
  }

  # make sure the order of columns is correct for the layer attributes
  layer_attributes <- layer_attributes %>% select("layer_id", "layer_name", everything())

  #Loop over the list of layers, creating an edge list for each network separately.
  for (layer_id in 1:length(list_of_layers)) {
    # get the layer network
    the_layer <- list_of_layers[[layer_id]]
    l_name <- layer_attributes[layer_id,'layer_name']
    message(sprintf('Layer #%s processing.', layer_id))

    ###  edge list  ###
    if (bipartite) {igraph_network <- create_monolayer_network(x = the_layer, directed = directed, bipartite = TRUE, group_names = c('set_cols','set_rows'))
                    edge_list <- igraph_network$edge_list
                    message('Done.')}
    if (!bipartite) {#Expected a symmetric matrix in an undirected network. Otherwise, throw the warning.
                     if ((directed == FALSE) && (isSymmetric(the_layer) ==FALSE)){
                       warning('WARNING: In an undirected network a symmetric matrix is expected. Proceed with caution!')
                     }
                     l_n_attrib <- state_node_attributes
                     if ('data.frame'%in%class(the_layer) & is.null(l_n_attrib)){ # no explicit node data provided
                        # get nodes from the edge list
                        nodes <- unique(c(the_layer$from, the_layer$to))

                        # get nodes from interlayer edge list
                        if (!is.null(interlayer_links)){
                          # filter links relevant to this layer
                          f_inter <- interlayer_links %>% filter(.data$layer_from == layer_id | .data$layer_from == l_name)
                          t_inter <- interlayer_links %>% filter(.data$layer_to == layer_id | .data$layer_to == l_name)
                          more_nodes <- unique(c(f_inter$node_from, t_inter$node_to)) # get layer nodes present in the interlayer edges

                          nodes <- unique(c(nodes, more_nodes))
                        }
                        l_n_attrib <- tibble(node_name = nodes, layer_name = layer_id)
                     } # find node data so we don't lose singletons
                     else if (!is.null(state_node_attributes)) {
                       l_n_attrib <- state_node_attributes %>% filter(.data$layer_name == l_name)
                     }

                     igraph_network <- create_monolayer_network(x = the_layer, directed = directed, bipartite = FALSE, node_metadata = l_n_attrib)
                     edge_list <- igraph_network$edge_list
                     message('Done.')
                     }

    #add the current edge list to the extended_edge_list
    ELL_layer <- edge_list
    ELL_layer$layer <- layer_to <- layer_attributes[layer_id,'layer_name']
    ELL_layer %<>% select(layer_from=layer, node_from=from, layer_to=layer, node_to=to, everything())
    extended_edge_list <- rbind(extended_edge_list,ELL_layer)

    ###  state nodes  ###
    state_nodes <- rbind(state_nodes, data.frame(layer_id=layer_id, node_name=igraph_network$nodes$node_name))
  }

  # Handle interlayer links
  if (!is.null(interlayer_links)){
    # If some nodes do not appear in the list of provided layers, interlayer_links will not be added to the extended edge list
    nodes_in_interlayer <- unique(c(interlayer_links$node_from, interlayer_links$node_to))
    nodes_in_layers <- unique(c(extended_edge_list$node_from, extended_edge_list$node_to))
    if (any(nodes_in_interlayer %in% nodes_in_layers)==FALSE){
      stop('STOPPING: Check node names in the interlayer_links dataframe. Some nodes do not appear in the list of provided layers')
    }
    # If some layers do not appear in the list of provided layers, interlayer_links will not be added to the extended edge list
    layers_in_interlayer <- unique(c(interlayer_links$layer_from, interlayer_links$layer_to))
    layers_in_intralayer <- unique(c(extended_edge_list$layer_from, extended_edge_list$layer_to))
    if (any(layers_in_interlayer %in% layers_in_intralayer)==FALSE){
      stop('STOPPING: Check layer names in the interlayer_links dataframe. Some layers do not appear in the list of provided layer attributes. You must provide a layer attributes table that include layer names.')
    }

    # Add the interlayer links
    extended_edge_list <- rbind(extended_edge_list, interlayer_links)
  }

  # This is necessary for making the IDs
  extended_edge_list <- as.data.frame(extended_edge_list)

  # Create physical nodes table
  physical_nodes <- unique(state_nodes$node_name)
  physical_nodes <- data.frame(node_id = 1:length(physical_nodes), node_name = sort(physical_nodes))
  # Add user-provided attributes
  if (!is.null(physical_node_attributes)){
    message('Organizing state nodes')
    suppressMessages(physical_nodes %<>% left_join(physical_node_attributes))
  }

  ###  conversion of the node names to node ids in the edge list  ###
  message('Creating extended link list with node IDs')
  extended_edge_list_ids <- extended_edge_list
  for (i in 1:nrow(extended_edge_list)) {
    extended_edge_list_ids[i,2] <- physical_nodes[physical_nodes$node_name == extended_edge_list[i,2],]$node_id
    extended_edge_list_ids[i,4] <- physical_nodes[physical_nodes$node_name == extended_edge_list[i,4],]$node_id
    # Change layer names for ids
    extended_edge_list_ids[i,1] <- layer_attributes[layer_attributes$layer_name == extended_edge_list[i,1],]$layer_id
    extended_edge_list_ids[i,3] <- layer_attributes[layer_attributes$layer_name == extended_edge_list[i,3],]$layer_id
  }

  # Make IDs integer instead of character
  # extended_edge_list_ids[1:4,] <- apply(extended_edge_list_ids[1:4,], MARGIN = 2, FUN = as.integer) # This does not work well when link attreibutes are provided.
  extended_edge_list_ids$layer_from <- as.integer(extended_edge_list_ids$layer_from)
  extended_edge_list_ids$node_from <- as.integer(extended_edge_list_ids$node_from)
  extended_edge_list_ids$layer_to <- as.integer(extended_edge_list_ids$layer_to)
  extended_edge_list_ids$node_to <- as.integer(extended_edge_list_ids$node_to)

  # Organized the layer_attributes and state_nodes
  message('Organizing state nodes')
  state_nodes <- as_tibble(state_nodes)
  suppressMessages(
  state_nodes %<>% left_join(layer_attributes) %>%
    left_join(as_tibble(physical_nodes)) %>%
    select("layer_id", "node_id", "layer_name", "node_name")
  )
  if (!is.null(state_node_attributes)){
    message('Joining with user-provided state nodes')
    state_nodes %<>% left_join(state_node_attributes)
  }

  # Output
  out <- list(nodes=as_tibble(physical_nodes),
              layers = as_tibble(layer_attributes),
              extended=as_tibble(extended_edge_list),
              extended_ids=as_tibble(extended_edge_list_ids),
              state_nodes=as_tibble(state_nodes),
              description=NULL,
              references=NULL)

  class(out) <- 'multilayer'

  return(out)
}


### examples ###

# x <- matrix(rbinom(n = 15, size = 1, prob = 0.5),nrow=3, ncol = 5)
# y <- matrix(rbinom(n = 15, size = 1, prob = 0.5),nrow=3, ncol = 5)
#
# matrix1 <- as.matrix(create_multilayer_network(list(x,y), bipartite = TRUE, directed = TRUE,
#                                                get_sam = TRUE))
# matrix2 <- as.matrix(create_multilayer_network(list(bipartite::olesen2002aigrettes,bipartite::olesen2002flores),
#                                                get_sam = TRUE, bipartite = TRUE, directed = FALSE, interlayer_links = data.frame(node_from = 'Protaetia.aurichalcea', layer_from = 1, node_to = 'Reseda:luteola', layer_to = 2, weight = 234)))
#
#
#
#
# image(x = matrix2, useRaster=TRUE, axes=FALSE)


#### QA ### Shirly
#
# x_binary_Nonames_square<-matrix(rbinom(16, 1, 0.6), 4, 4)
# x_Nobinary_Nonames_square <- x_binary_Nonames_square*round(runif(16,1,4),0)
#
# x_Nobinary_names_square<-x_Nobinary_Nonames_square
# rownames(x_Nobinary_names_square)<-c('Orange', 'Blue', 'Red', 'Green')
# colnames(x_Nobinary_names_square)<-c('Orange', 'Blue', 'Red', 'Green')
#
# x2_Nobinary_names_square<-x_Nobinary_Nonames_square
# rownames(x2_Nobinary_names_square)<-c('Red', 'Purple', 'Green', 'White')
# colnames(x2_Nobinary_names_square)<-c('Red', 'Purple', 'Green', 'White')
#
# x3_Nobinary_names_square<-x_Nobinary_Nonames_square
# rownames(x3_Nobinary_names_square)<-c('Orange', 'Purple', 'Red', 'White')
# colnames(x3_Nobinary_names_square)<-c('Orange', 'Purple', 'Red', 'White')
#
#
# xy_Nobinary_namesColRow_square<-x_Nobinary_names_square
# colnames(xy_Nobinary_namesColRow_square) <-c("Xx", "Gg","Aa","Bb")

# Unipartite directed matrices: A->B, B->C in layer 1; A->C, B->A, C->B in layer 2
mat1 <- matrix(
  c(0, 1, 0,
    0, 0, 1,
    0, 0, 0),
  nrow = 3, byrow = TRUE,
  dimnames = list(c("A", "B", "C"), c("A", "B", "C"))
)
mat2 <- matrix(
  c(0, 0, 1,
    1, 0, 0,
    0, 1, 0),
  nrow = 3, byrow = TRUE,
  dimnames = list(c("A", "B", "C"), c("A", "B", "C"))
)

# Bipartite matrices: plants x pollinators
bip1 <- matrix(
  c(1, 0, 1,
    0, 1, 1),
  nrow = 2, byrow = TRUE,
  dimnames = list(c("Plant1", "Plant2"), c("Bee1", "Bee2", "Bee3"))
)
bip2 <- matrix(
  c(0, 1, 1,
    1, 0, 1),
  nrow = 2, byrow = TRUE,
  dimnames = list(c("Plant1", "Plant2"), c("Bee1", "Bee2", "Bee3"))
)

test_that("create_multilayer_network returns a multilayer object", {
  result <- create_multilayer_network(list(mat1, mat2),
                                      bipartite = FALSE, directed = TRUE)
  expect_s3_class(result, "multilayer")
})

test_that("create_multilayer_network has all required element names", {
  result <- create_multilayer_network(list(mat1, mat2),
                                      bipartite = FALSE, directed = TRUE)
  expect_named(result, c("nodes", "layers", "extended", "extended_ids",
                          "state_nodes", "description", "references"))
})

test_that("create_multilayer_network layers table has correct row count", {
  result <- create_multilayer_network(list(mat1, mat2),
                                      bipartite = FALSE, directed = TRUE)
  expect_equal(nrow(result$layers), 2)
})

test_that("create_multilayer_network nodes contains all physical nodes", {
  result <- create_multilayer_network(list(mat1, mat2),
                                      bipartite = FALSE, directed = TRUE)
  expect_equal(nrow(result$nodes), 3)
  expect_setequal(result$nodes$node_name, c("A", "B", "C"))
})

test_that("create_multilayer_network auto-generates layer_id and layer_name", {
  result <- create_multilayer_network(list(mat1, mat2),
                                      bipartite = FALSE, directed = TRUE)
  expect_true("layer_id"   %in% names(result$layers))
  expect_true("layer_name" %in% names(result$layers))
})

test_that("create_multilayer_network uses custom layer attributes", {
  layer_attrib <- data.frame(layer_id = 1:2,
                              layer_name = c("pond_1", "pond_2"))
  result <- create_multilayer_network(list(mat1, mat2),
                                      bipartite = FALSE, directed = TRUE,
                                      layer_attributes = layer_attrib)
  expect_equal(result$layers$layer_name, c("pond_1", "pond_2"))
})

test_that("create_multilayer_network extended edge list has required columns", {
  result <- create_multilayer_network(list(mat1, mat2),
                                      bipartite = FALSE, directed = TRUE)
  expect_true(all(c("layer_from", "node_from", "layer_to", "node_to", "weight") %in%
                    names(result$extended)))
})

test_that("create_multilayer_network extended_ids has integer layer and node IDs", {
  result <- create_multilayer_network(list(mat1, mat2),
                                      bipartite = FALSE, directed = TRUE)
  expect_true(is.integer(result$extended_ids$layer_from))
  expect_true(is.integer(result$extended_ids$node_from))
})

test_that("create_multilayer_network interlayer links are added to extended edge list", {
  layer_attrib <- data.frame(layer_id = 1:2,
                              layer_name = c("layer_1", "layer_2"))
  interlayer <- data.frame(
    layer_from = "layer_1", node_from = "A",
    layer_to   = "layer_2", node_to   = "A",
    weight     = 1
  )
  result <- create_multilayer_network(list(mat1, mat2),
                                      bipartite = FALSE, directed = TRUE,
                                      layer_attributes = layer_attrib,
                                      interlayer_links = interlayer)
  inter_rows <- result$extended[result$extended$layer_from != result$extended$layer_to, ]
  expect_gt(nrow(inter_rows), 0)
})

test_that("create_multilayer_network works with bipartite matrices", {
  result <- create_multilayer_network(list(bip1, bip2),
                                      bipartite = TRUE, directed = FALSE)
  expect_s3_class(result, "multilayer")
  expect_equal(nrow(result$layers), 2)
  # All plants and bees are physical nodes
  expect_setequal(result$nodes$node_name,
                  c("Plant1", "Plant2", "Bee1", "Bee2", "Bee3"))
})

test_that("create_multilayer_network errors when first column is not layer_id", {
  wrong_attrib <- data.frame(id = 1:2, layer_name = c("p1", "p2"))
  expect_error(
    create_multilayer_network(list(mat1, mat2),
                               bipartite = FALSE, directed = TRUE,
                               layer_attributes = wrong_attrib)
  )
})

test_that("create_multilayer_network errors on mismatched layer count", {
  layer_attrib_3 <- data.frame(layer_id = 1:3,
                                 layer_name = c("l1", "l2", "l3"))
  expect_error(
    create_multilayer_network(list(mat1, mat2),
                               bipartite = FALSE, directed = TRUE,
                               layer_attributes = layer_attrib_3)
  )
})

test_that("create_multilayer_network errors when layer_name column has duplicates", {
  dup_attrib <- data.frame(layer_id = 1:2, layer_name = c("same", "same"))
  expect_error(
    create_multilayer_network(list(mat1, mat2),
                               bipartite = FALSE, directed = TRUE,
                               layer_attributes = dup_attrib)
  )
})

#' Adjacency matrix to edge list conversion
#'
#' Convert an adjacency matrix to an edge list.
#'
#' @details Assigns column and row names if those do not exist. Cannot handle node attributes/metadata.
#'
#' @param x An adjacency or incidence matrix.
#' @param directed Is the network directed? TRUE/FALSE
#'
#' @return A \code{monolayer} object
#'
#' @seealso \code{create_monolayer_network, monolayer}
#'
#' @examples
#' # See examples in:
#' # https://emln.ecomplab.com/monolayer.html#Unipartite
#'
#' # Generate a matrix with random weighted interactions
#' x <- matrix(rbinom(100,1,0.6),10,10)
#' x <- x*round(runif(100,1,5),0)
#' # run
#' matrix_to_list_unipartite(x, directed = TRUE)
#'
#' @export
#' @import dplyr
#' @importFrom igraph is_directed graph_from_adjacency_matrix degree

matrix_to_list_unipartite <- function(x, directed){
  # Assign column and row names if missing
  if (is.null(rownames(x)) && is.null(colnames(x))) {message('Assigning missing row and column names'); rownames(x) <- colnames(x) <- paste('X',1:nrow(x),sep='')}
  if (is.null(rownames(x)) && !is.null(colnames(x))) {message('Rows renamed as columns'); rownames(x) <- colnames(x)}
  if (!is.null(rownames(x)) && is.null(colnames(x))) {message('Columns renamed as rows'); colnames(x) <- rownames(x)}
  # Or make sure they are identical
  if(!identical(rownames(x),colnames(x))){message('Rows and columns do not have the same names or are not in the same order! Proceed with caution!')}

  g <- igraph::graph_from_adjacency_matrix(t(x), weighted = TRUE, mode = ifelse(directed, 'directed','undirected')) # For some reason igraph considers the from to be ther rows. Need to transpose the matrix
  # summary(g)
  if(any(igraph::degree(g)==0)){message('Some nodes have no interactions. They will appear in the node table but not in the edge list')}
  l_unip <- as_tibble(igraph::as_data_frame(g, 'edges'))

  nodes <- rownames(x)
  node_list <- tibble(node_id=1:length(nodes),node_name=nodes)
  out <- list(mode='U', directed=igraph::is_directed(g), nodes=node_list, mat=x, edge_list=l_unip, igraph_object=g)
  class(out) <- "monolayer"
  return(out)
}

mat_directed <- matrix(
  c(0, 1, 0,
    0, 0, 1,
    1, 0, 0),
  nrow = 3, byrow = TRUE,
  dimnames = list(c("A", "B", "C"), c("A", "B", "C"))
)

mat_undirected <- matrix(
  c(0, 1, 1,
    1, 0, 1,
    1, 1, 0),
  nrow = 3, byrow = TRUE,
  dimnames = list(c("A", "B", "C"), c("A", "B", "C"))
)

mat_nonames <- matrix(
  c(0, 1, 0,
    0, 0, 1,
    1, 0, 0),
  nrow = 3, byrow = TRUE
)

test_that("matrix_to_list_unipartite returns a monolayer object", {
  result <- matrix_to_list_unipartite(mat_directed, directed = TRUE)
  expect_s3_class(result, "monolayer")
})

test_that("matrix_to_list_unipartite sets mode to U", {
  result <- matrix_to_list_unipartite(mat_directed, directed = TRUE)
  expect_equal(result$mode, "U")
})

test_that("matrix_to_list_unipartite reports directed correctly", {
  result_dir <- matrix_to_list_unipartite(mat_directed, directed = TRUE)
  result_und <- matrix_to_list_unipartite(mat_undirected, directed = FALSE)
  expect_true(result_dir$directed)
  expect_false(result_und$directed)
})

test_that("matrix_to_list_unipartite has all required list elements", {
  result <- matrix_to_list_unipartite(mat_directed, directed = TRUE)
  expect_named(result, c("mode", "directed", "nodes", "mat", "edge_list", "igraph_object"))
})

test_that("matrix_to_list_unipartite nodes table has correct columns and row count", {
  result <- matrix_to_list_unipartite(mat_directed, directed = TRUE)
  expect_true(all(c("node_id", "node_name") %in% names(result$nodes)))
  expect_equal(nrow(result$nodes), 3)
})

test_that("matrix_to_list_unipartite node names match matrix row/col names", {
  result <- matrix_to_list_unipartite(mat_directed, directed = TRUE)
  expect_setequal(result$nodes$node_name, rownames(mat_directed))
})

test_that("matrix_to_list_unipartite edge list has from, to, weight columns", {
  result <- matrix_to_list_unipartite(mat_directed, directed = TRUE)
  expect_true(all(c("from", "to", "weight") %in% names(result$edge_list)))
})

test_that("matrix_to_list_unipartite preserves the input matrix", {
  result <- matrix_to_list_unipartite(mat_directed, directed = TRUE)
  expect_equal(result$mat, mat_directed)
})

test_that("matrix_to_list_unipartite assigns names when both are missing", {
  result <- matrix_to_list_unipartite(mat_nonames, directed = TRUE)
  expect_s3_class(result, "monolayer")
  expect_equal(nrow(result$nodes), 3)
  expect_false(is.null(result$nodes$node_name))
})

test_that("matrix_to_list_unipartite igraph_object is an igraph", {
  result <- matrix_to_list_unipartite(mat_directed, directed = TRUE)
  expect_true(inherits(result$igraph_object, "igraph"))
})

#'
#' @docType data
#' @title emln46_spatial_p_jordano_unpubl
#' 
#' @description Network Description:
#' 
#' | network_id | multilayer_network_type | ecological_network_type | state_nodes |
#' |:----------:|:-----------------------:|:-----------------------:|:-----------:|
#' |     46     |         Spatial         |     Seed-Dispersal      |    FALSE    |
#' 
#' @format NULL
#' @usage NULL
#' @source Web_of_Life
#' @source http://www.web-of-life.es/map.php
#' @md
#' @keywords internal
'emln46_spatial_p_jordano_unpubl'

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/emln64_temporal_caradona_2020.R
\docType{data}
\name{emln64_temporal_caradona_2020}
\alias{emln64_temporal_caradona_2020}
\title{emln64_temporal_caradona_2020}
\source{
EDI_data_portal

https://portal.edirepository.org/nis/mapbrowse?packageid=edi.512.1
}
\description{
Network Description:\tabular{cccc}{
   network_id \tab multilayer_network_type \tab ecological_network_type \tab state_nodes \cr
   64 \tab Temporal \tab Pollination \tab FALSE \cr
}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/matrix_to_list_unipartite.R
\name{matrix_to_list_unipartite}
\alias{matrix_to_list_unipartite}
\title{Adjacency matrix to edge list conversion}
\usage{
matrix_to_list_unipartite(x, directed)
}
\arguments{
\item{x}{An adjacency or incidence matrix.}

\item{directed}{Is the network directed? TRUE/FALSE}
}
\value{
A \code{monolayer} object
}
\description{
Convert an adjacency matrix to an edge list.
}
\details{
Assigns column and row names if those do not exist. Cannot handle node attributes/metadata.
}
\examples{
# See examples in:
# https://emln.ecomplab.com/monolayer.html#Unipartite

# Generate a matrix with random weighted interactions
x <- matrix(rbinom(100,1,0.6),10,10)
x <- x*round(runif(100,1,5),0)
# run
matrix_to_list_unipartite(x, directed = TRUE)

}
\seealso{
\code{create_monolayer_network, monolayer}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/emln69_temporal_fang_2012.R
\docType{data}
\name{emln69_temporal_fang_2012}
\alias{emln69_temporal_fang_2012}
\title{emln69_temporal_fang_2012}
\source{
web

https://figshare.com/articles/dataset/Relative_Stability_of_Core_Groups_in_Pollination_Networks_in_a_Biodiversity_Hotspot_over_Four_Years/127797
}
\description{
Network Description:\tabular{cccc}{
   network_id \tab multilayer_network_type \tab ecological_network_type \tab state_nodes \cr
   69 \tab Temporal \tab Pollination \tab FALSE \cr
}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/multilayer_to_json.R
\name{.df_to_list_of_rows}
\alias{.df_to_list_of_rows}
\title{Convert a data.frame to a list of named lists (one per row)}
\usage{
.df_to_list_of_rows(df)
}
\description{
Convert a data.frame to a list of named lists (one per row)
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/emln1_environment_schleuning_2010.R
\docType{data}
\name{emln1_environment_schleuning_2010}
\alias{emln1_environment_schleuning_2010}
\title{emln1_environment_schleuning_2010}
\source{
Interaction_Web_DataBase

http://www.ecologia.ib.usp.br/iwdb/html/schleuning-et-al-2010.html
}
\description{
Network Description:\tabular{cccc}{
   network_id \tab multilayer_network_type \tab ecological_network_type \tab state_nodes \cr
   1 \tab Environment \tab Seed-Dispersal \tab FALSE \cr
}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/emln49_spatial_perturbation_vazquez_2002.R
\docType{data}
\name{emln49_spatial_perturbation_vazquez_2002}
\alias{emln49_spatial_perturbation_vazquez_2002}
\title{emln49_spatial_perturbation_vazquez_2002}
\source{
Interaction_Web_DataBase

http://www.ecologia.ib.usp.br/iwdb/html/vazquez_2002.html
}
\description{
Network Description:\tabular{cccc}{
   network_id \tab multilayer_network_type \tab ecological_network_type \tab state_nodes \cr
   49 \tab Spatial     , Perturbation \tab Pollination \tab FALSE \cr
}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/emln60_spatial_trojelsgaard_2015.R
\docType{data}
\name{emln60_spatial_trojelsgaard_2015}
\alias{emln60_spatial_trojelsgaard_2015}
\title{emln60_spatial_trojelsgaard_2015}
\source{
Dryad

https://datadryad.org/stash/dataset/doi:10.5061/dryad.76173
}
\description{
Network Description:\tabular{cccc}{
   network_id \tab multilayer_network_type \tab ecological_network_type \tab state_nodes \cr
   60 \tab Spatial \tab Pollination \tab FALSE \cr
}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_multilayer.R
\name{plot_multilayer}
\alias{plot_multilayer}
\title{Plot a multilayer network in the browser}
\usage{
plot_multilayer(
  multilayer,
  bipartite = FALSE,
  setA_type = NULL,
  directed = NULL,
  directed_interlayer = NULL,
  port = 8080,
  mira_path = NULL,
  browser = getOption("browser")
)
}
\arguments{
\item{multilayer}{A multilayer object (created by \code{create_multilayer_network}
or \code{load_emln}).}

\item{bipartite}{Logical. Whether the network is bipartite. Defaults to
\code{FALSE}. Bipartite is not auto-detected. When \code{TRUE}, the nodes
table must contain a \code{node_type} (or legacy \code{node_group})
column with exactly two distinct values across the network.}

\item{setA_type}{Optional character. The \code{node_type} value to render
as Set A (top row) in MiRA's bipartite layout. By ecological convention
this is the higher trophic level (e.g. \code{"pollinator"},
\code{"parasite"}, \code{"disperser"}). Only used when
\code{bipartite = TRUE}. Forwarded to \code{multilayer_to_json}.}

\item{directed}{Logical or NULL. Whether intralayer links are directed.
If NULL (default), auto-detected by checking intralayer edge symmetry.}

\item{directed_interlayer}{Logical or NULL. Whether interlayer links are
directed. If NULL (default), inherits from \code{directed}.}

\item{port}{Integer. Port for the local HTTP server. Default is 8080.}

\item{mira_path}{Character. Path to the MiRA web-app directory. If NULL
(default), uses the copy bundled with the package
(\code{system.file("MiRA", package = "emln")}).}

\item{browser}{Choose the specific browser to open MiRA in. Defaults to
the system default via \code{getOption("browser")}. Can be a browser
name (e.g. "chrome", "firefox") or a command
(e.g. "open -a 'Google Chrome'").}
}
\value{
Invisibly returns the server handle. Use
  \code{httpuv::stopServer(handle)} to stop the server when done.
}
\description{
Converts an EMLN \code{multilayer} object to JSON and opens MiRA, the
multilayer interactive R-network app, in the default web browser.
}
\details{
The function:
\enumerate{
  \item Converts the multilayer object to JSON via \code{multilayer_to_json}
  \item Starts a local HTTP server (using \code{httpuv}) that serves the
    MiRA app and the network JSON
  \item Opens the browser with auto-load enabled
}

The server runs in the background. Call \code{httpuv::stopServer(handle)} or
close R to stop it. The JSON data is kept in memory and never written to disk.
}
\note{
MiRA (Multilayer Interactive Rendering Application) is a browser-based
interactive visualization tool for multilayer networks, available at
\url{https://mira.ecomplab.com/}.

If you use MiRA in published research, please cite:

Nehoray SM, Bloch Y, Pilosof S (2026). Interactively visualizing biological
multilayer networks using MiRA. \emph{arXiv}:2605.09597 [cs.SI].
\doi{doi:10.48550/arXiv.2605.09597}

Feedback and bug reports:
\url{https://github.com/Ecological-Complexity-Lab/MiRA/issues}.
}
\examples{
if (interactive()){
net <- load_emln(60)
srv <- plot_multilayer(net, bipartite = TRUE, setA_type = "pollinator")

# When done:
httpuv::stopServer(srv)
}
}
\seealso{
\code{multilayer_to_json, create_multilayer_network, load_emln}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/emln17_multiplex_traveset_2020.R
\docType{data}
\name{emln17_multiplex_traveset_2020}
\alias{emln17_multiplex_traveset_2020}
\title{emln17_multiplex_traveset_2020}
\source{
Figshare

https://doi.org/10.6084/m9.figshare.c.5208546.v1
}
\description{
Network Description:\tabular{cccc}{
   network_id \tab multilayer_network_type \tab ecological_network_type \tab state_nodes \cr
   17 \tab Multiplex \tab Multiple \tab FALSE \cr
}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/multilayer_to_json.R
\name{.to_json}
\alias{.to_json}
\title{Minimal JSON serializer (no dependency on jsonlite)}
\usage{
.to_json(x, indent = 0)
}
\description{
Converts an R list structure to a JSON string.
Supports: lists, vectors, strings, numbers, logicals, NULL.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/emln23_spatial_canadian_fish_parasite.R
\docType{data}
\name{emln23_spatial_canadian_fish_parasite}
\alias{emln23_spatial_canadian_fish_parasite}
\title{emln23_spatial_canadian_fish_parasite}
\source{
Interaction_Web_DataBase

http://www.ecologia.ib.usp.br/iwdb/html/canadian_fish.html
}
\description{
Network Description:\tabular{cccc}{
   network_id \tab multilayer_network_type \tab ecological_network_type \tab state_nodes \cr
   23 \tab Spatial \tab Host-Parasite \tab TRUE \cr
}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/emln40_spatial_medan_2002.R
\docType{data}
\name{emln40_spatial_medan_2002}
\alias{emln40_spatial_medan_2002}
\title{emln40_spatial_medan_2002}
\source{
Web_of_Life

Interaction_Web_DataBase

http://www.ecologia.ib.usp.br/iwdb/html/medan_et_al_2002.html

http://www.web-of-life.es/map.php
}
\description{
Network Description:\tabular{cccc}{
   network_id \tab multilayer_network_type \tab ecological_network_type \tab state_nodes \cr
   40 \tab Spatial \tab Pollination \tab FALSE \cr
}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/create_multilayer_network.R
\name{create_multilayer_network}
\alias{create_multilayer_network}
\title{Create Multilayer Network}
\usage{
create_multilayer_network(
  list_of_layers,
  bipartite,
  directed,
  interlayer_links = NULL,
  layer_attributes = NULL,
  state_node_attributes = NULL,
  physical_node_attributes = NULL
)
}
\arguments{
\item{list_of_layers}{List of matrices or list of link lists (format
\code{from to weight}; see description). These are the intralayer links.}

\item{bipartite}{Is the network bipartite? Must be provided (no default
value).}

\item{directed}{Is the network directed? Must be provided (no default value).}

\item{interlayer_links}{Interlayer extended edge list of the format
\code{layer_from node_from layer_to node_to weight}. Default is NULL,
assuming no interlayer links.}

\item{layer_attributes}{Optional. A data frame with layer attributes. The
first column must be called \code{layer_id}, and it must have unique values (one row per layer).
Layer name column, if exists must be called \code{layer_name}, and its values must be unique. The
order of the rows and layer_id should be the same as in \code{list_of_layers}.
No column is allowed to be named \code{name}.}

\item{state_node_attributes}{Optional. Additional information on physical nodes in
layers. Must contain columns \code{layer_name} and \code{node_name}.}

\item{physical_node_attributes}{Optional. Additional information on physical nodes. Must contain column \code{node_name}.}
}
\value{
A multilayer object (see ?multilayer).
}
\description{
This function creates a multilayer network from multiple matrices / link
lists.
}
\details{
Input of \code{list_of_layers} is handled by the function
  \code{create_monolayer_network}; see it's description for details on input.
  If using matrices as input, they should contain row and column names. If link lists are provided as input column names should be
  \code{from to weight}.

  When using link lists, it is possible to include link
  attributes. In that case the headers of all the link lists and, if
  included, the interlayer links, must be the same (after the weight), even if some attributes
  are included in only some layers (see detailed example in the code accompanying the package). Missing link attributes can be set to \code{NA}.

  If interlayer links are provided, then layer_attributes must also be
  provided. Specify layer and nodes by UNIQUE names (not IDs). Layer names should
  correspond to those provided in layer_attributes.

  Attributes of state nodes can be provided with \code{state_node_attributes}. If
  provided, then columns \code{layer_name} and \code{node_name} must be
  included.

  For compatibility with function \code{load_emln} the output contains
  \code{description} and \code{references}. The value of any missing data
  frames in the multilayer object will be set to NULL.

  See multiple example in the code accompanying the package
}
\examples{
library(tibble)
pond_1 <- matrix(c(0,1,1,0,0,1,0,0,0), byrow = TRUE,
nrow = 3, ncol = 3, dimnames = list(c('pelican','fish','crab'),
c('pelican','fish','crab')))
pond_2 <- matrix(c(0,1,0,0,0,1,0,0,0), byrow = TRUE,
nrow = 3, ncol = 3, dimnames = list(c('pelican','fish','crab'),
c('pelican','fish','crab')))
pond_3 <- matrix(c(0,1,1,0,0,1,0,0,0), byrow = TRUE,
nrow = 3, ncol = 3, dimnames = list(c('pelican','fish','tadpole'),
c('pelican','fish','tadpole')))

layer_attrib <- tibble(layer_id=1:3,
                       layer_name=c('pond_1','pond_2','pond_3'),
                       location=c('valley','mid-elevation','uphill'))

# Create the ELL tibble with interlayer links.
interlayer <- tibble(layer_from=c('pond_1','pond_1','pond_1'),
                     node_from=c('pelican','crab','pelican'),
                     layer_to=c('pond_2','pond_2','pond_3'),
                    node_to=c('pelican','crab','pelican'),
                     weight=1)

# This is a directed network so the links should go both ways,
# even though they are symmetric.
interlayer_2 <- interlayer[,c(3,4,1,2,5)]
names(interlayer_2) <- names(interlayer)
interlayer <- rbind(interlayer, interlayer_2)

multilayer <- create_multilayer_network(list_of_layers =
list(pond_1, pond_2, pond_3), layer_attributes = layer_attrib,
interlayer_links = interlayer, bipartite = FALSE, directed = TRUE)

}
\seealso{
\code{multilayer, create_monolayer_network}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/emln39_spatial_leather_1991.R
\docType{data}
\name{emln39_spatial_leather_1991}
\alias{emln39_spatial_leather_1991}
\title{emln39_spatial_leather_1991}
\source{
Web_of_Life

Interaction_Web_DataBase

http://www.web-of-life.es/map.php

http://www.ecologia.ib.usp.br/iwdb/html/leather_1991.htm
}
\description{
Network Description:\tabular{cccc}{
   network_id \tab multilayer_network_type \tab ecological_network_type \tab state_nodes \cr
   39 \tab Spatial \tab Plant-Herbivore \tab TRUE \cr
}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/emln35_spatial_ingversen_2006.R
\docType{data}
\name{emln35_spatial_ingversen_2006}
\alias{emln35_spatial_ingversen_2006}
\title{emln35_spatial_ingversen_2006}
\source{
Web_of_Life

http://www.web-of-life.es/map.php
}
\description{
Network Description:\tabular{cccc}{
   network_id \tab multilayer_network_type \tab ecological_network_type \tab state_nodes \cr
   35 \tab Spatial \tab Pollination \tab FALSE \cr
}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/create_monolayer_network.R
\name{create_monolayer_network}
\alias{create_monolayer_network}
\title{Creates a monolayer network object.}
\usage{
create_monolayer_network(
  x,
  directed = NULL,
  bipartite = NULL,
  group_names = c("set_cols", "set_rows"),
  node_metadata = NULL
)
}
\arguments{
\item{x}{input data in the format of a matrix, edge list or an igraph object.}

\item{directed}{Is the network directed?}

\item{bipartite}{Is the network bipartite? Ignored when the input is an igraph object.}

\item{group_names}{For bipartite networks: name of the groups in the columns
and rows, respectively (e.g., parasites and hosts).}

\item{node_metadata}{Following the igraph method of \code{graph_from_data_frame}.
First column must be have names matching those in x.}
}
\value{
A \code{monolayer} object.
}
\description{
Automatically identifies if the input is a matrix or an edge list and creates
a \code{monolayer} object.
}
\details{
Converts between edge list and matrix formats and creates a
  monolayer object. It a wrapper function for
  \code{list_to_matrix, matrix_to_list_unipartite, matrix_to_list_bipartite}. Node metadata can only be included with an edge list input.
}
\examples{
library(igraph)

# A bipartite network from package bipartte
x <- create_monolayer_network(bipartite::memmott1999, bipartite = TRUE,
directed = FALSE, group_names = c('Animals','Plants'))

# A bipartite network as an igraph object

# Generate a random bipartite network in igraph
g <- igraph::sample_bipartite(10,16,p=0.3, type = 'gnp', directed = TRUE, mode = 'in')
V(g)$name <- letters # name the nodes
V(g)$gender <- sample(c('F','M'), size = 26, replace = TRUE) # Add a node attribute
E(g)$weight=runif(ecount(g)) # Add edge weights
plot(g, layout=layout.bipartite)
create_monolayer_network(g, group_names = c('Parasites','Hosts'))




}
\seealso{
\code{list_to_matrix, matrix_to_list_unipartite, monolayer}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/emln25_spatial_carstensen_2014.R
\docType{data}
\name{emln25_spatial_carstensen_2014}
\alias{emln25_spatial_carstensen_2014}
\title{emln25_spatial_carstensen_2014}
\source{
figshare

https://figshare.com/articles/dataset/\emph{Beta_Diversity_of_Plant_Pollinator_Networks_and_the_Spatial_Turnover_of_Pairwise_Interactions}/1234708
}
\description{
Network Description:\tabular{cccc}{
   network_id \tab multilayer_network_type \tab ecological_network_type \tab state_nodes \cr
   25 \tab Spatial \tab Pollination \tab FALSE \cr
}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/emln63_temporal_brodeur_1992.R
\docType{data}
\name{emln63_temporal_brodeur_1992}
\alias{emln63_temporal_brodeur_1992}
\title{emln63_temporal_brodeur_1992}
\source{
mangal

https://mangal.io/doc/api/
}
\description{
Network Description:\tabular{ccccc}{
   network_id \tab multilayer_network_type \tab mangal_code \tab ecological_network_type \tab state_nodes \cr
   63 \tab Temporal \tab 55 \tab Food-Web \tab FALSE \cr
}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/emln2_environment_benadi_jae_2013.R
\docType{data}
\name{emln2_environment_benadi_jae_2013}
\alias{emln2_environment_benadi_jae_2013}
\title{emln2_environment_benadi_jae_2013}
\source{
Dryad

https://datadryad.org/stash/dataset/doi:10.5061/dryad.8mn44
}
\description{
Network Description:\tabular{cccc}{
   network_id \tab multilayer_network_type \tab ecological_network_type \tab state_nodes \cr
   2 \tab Environment \tab Pollination \tab FALSE \cr
}
}
\keyword{internal}

\name{multilayer class}
\alias{multilayer}
\title{An object of class multilayer}

\value{
A list with:
\itemize{
   \item \code{nodes} Physical nodes. First column is a unique node id. Node attributes are included if provided as input.
   \item \code{layers} Information on layers.
   \item \code{extended} An extended link list of the format \code{layer_from node_from layer_to node_to weight}. All nodes and layers are identified by names.
   \item \code{extended_ids} An extended link list of the format \code{layer_from node_from layer_to node_to weight}. All nodes and layers are identified by unique IDs automoatically generated.
   \item \code{state_nodes} List of state nodes of the format \code{layer_id node_id layer_name node_name}. Also includes state node attributes if provided as input.
   \item \code{description} For compatibility with \code{load_emln}.
   \item \code{references} For compatibility with \code{load_emln}.
}
}
\description{
A network object of class multilayer contains all the necessary information and R objects that define a multilayer network.
}
\seealso{
\code{create_multilayer_network}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/emln50_spatial_poulin_1999.R
\docType{data}
\name{emln50_spatial_poulin_1999}
\alias{emln50_spatial_poulin_1999}
\title{emln50_spatial_poulin_1999}
\source{
Web_of_Life

Interaction_Web_DataBase

http://www.web-of-life.es/map.php

http://www.ecologia.ib.usp.br/iwdb/resources.html
}
\description{
Network Description:\tabular{cccc}{
   network_id \tab multilayer_network_type \tab ecological_network_type \tab state_nodes \cr
   50 \tab Spatial \tab Seed-Dispersal \tab FALSE \cr
}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/load_emln.R
\name{load_emln}
\alias{load_emln}
\title{Load networks include din the package}
\usage{
load_emln(network_id)
}
\arguments{
\item{network_id}{The network ID.}
}
\value{
A multilayer object (see ?multilayer).
}
\description{
Load a specific network by specifying its ID.
}
\details{
Use the \code{search_emln} and \code{view_emln} functions to obtain network IDs.
}
\examples{
\donttest{
load_emln(network_id = 38)
}

}
\seealso{
\code{multilayer}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/emln33_spatial_hadfield_2013.R
\docType{data}
\name{emln33_spatial_hadfield_2013}
\alias{emln33_spatial_hadfield_2013}
\title{emln33_spatial_hadfield_2013}
\source{
Web_of_Life

Dryad

http://www.web-of-life.es/map.php

https://datadryad.org/stash/dataset/doi:10.5061/dryad.jf3tj
}
\description{
Network Description:\tabular{cccc}{
   network_id \tab multilayer_network_type \tab ecological_network_type \tab state_nodes \cr
   33 \tab Spatial \tab Host-Parasite \tab TRUE \cr
}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/emln53_spatial_slovakia.R
\docType{data}
\name{emln53_spatial_slovakia}
\alias{emln53_spatial_slovakia}
\title{emln53_spatial_slovakia}
\source{
NA

NA
}
\description{
Network Description:\tabular{cccc}{
   network_id \tab multilayer_network_type \tab ecological_network_type \tab state_nodes \cr
   53 \tab Spatial \tab Host-Parasite \tab FALSE \cr
}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/view_emln.R
\name{view_emln}
\alias{view_emln}
\title{View networks}
\usage{
view_emln()
}
\value{
An interactive GUI for browinsg through networks.
}
\description{
View the database interactively.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/multilayer_to_csv.R
\name{multilayer_to_csv}
\alias{multilayer_to_csv}
\title{Export a multilayer network to CSV files}
\usage{
multilayer_to_csv(
  multilayer,
  dir,
  prefix = "network",
  bipartite = FALSE,
  setA_type = NULL,
  directed = NULL
)
}
\arguments{
\item{multilayer}{A multilayer object (created by
\code{create_multilayer_network} or \code{load_emln}).}

\item{dir}{Directory to write the CSV files to. Created if it does not
already exist.}

\item{prefix}{Character. File name prefix. Defaults to \code{"network"}.
Three files are always written:
\code{<prefix>_edges.csv}, \code{<prefix>_layers.csv},
\code{<prefix>_nodes.csv}.
A fourth file \code{<prefix>_state_nodes.csv} is written when the
multilayer object contains per-(layer, node) attributes beyond
\code{layer_name} and \code{node_name} (e.g. abundance,
module).}

\item{bipartite}{Logical. Whether the network is bipartite. Defaults to
\code{FALSE}. When \code{TRUE}, the nodes table must contain a
\code{node_type} (or legacy \code{node_group}) column with exactly two
distinct values across the network, and a \code{bipartite} column is
written to the layers CSV.}

\item{setA_type}{Optional character. The \code{node_type} value to render
as Set A (top row) in MiRA's bipartite layout. By ecological convention
this is the higher trophic level (e.g. \code{"pollinator"},
\code{"parasite"}, \code{"disperser"}). Only used when
\code{bipartite = TRUE}; written as a \code{setA_type} column in the
layers CSV. If \code{NULL} (default), MiRA falls back to alphabetical
ordering of the two types.}

\item{directed}{Logical or NULL. Whether the network is directed. If NULL
(default), auto-detected by checking intralayer edge symmetry.}
}
\value{
Invisibly returns a named character vector of file paths written
  (\code{edges}, \code{layers}, \code{nodes}, and optionally
  \code{state_nodes}).
}
\description{
Writes an EMLN \code{multilayer} object as a set of CSV files consumed
by MiRA's CSV importer. Three files are always written; a fourth
(\code{<prefix>_state_nodes.csv}) is added when per-(layer, node)
attributes are present.
}
\details{
The CSV files follow the schema accepted by MiRA's CSV importer:
\itemize{
  \item \strong{edges}: \code{layer_from, node_from, layer_to, node_to,
    weight} (+ any extra link attributes).
  \item \strong{layers}: \code{layer_id, layer_name} (+ \code{latitude,
    longitude, bipartite} and any extra layer attributes).
  \item \strong{nodes}: \code{node_name} (+ \code{node_type} and any extra
    physical node attributes, same across all layers).
    \code{node_group} is renamed to \code{node_type}.
  \item \strong{state_nodes} (written only when extra per-(layer, node)
    attributes exist): \code{layer_name, node_name} + extra columns.
    Use this when the same node has different attribute values in
    different layers (e.g. abundance, module membership).
}
The \code{directed} flag is not written to the CSV files; it is a property
you specify in MiRA's CSV import dialog at load time.
}
\note{
MiRA (Multilayer Interactive Rendering Application) is a browser-based
interactive visualization tool for multilayer networks, available at
\url{https://mira.ecomplab.com/}.

If you use MiRA in published research, please cite:

Nehoray SM, Bloch Y, Pilosof S (2026). Interactively visualizing biological
multilayer networks using MiRA. \emph{arXiv}:2605.09597 [cs.SI].
\doi{doi:10.48550/arXiv.2605.09597}

Feedback and bug reports:
\url{https://github.com/Ecological-Complexity-Lab/MiRA/issues}.
}
\examples{
\donttest{
net <- load_emln(14)
multilayer_to_csv(net, dir = "tests/out/", prefix = "kefi", bipartite = TRUE)
}
}
\seealso{
\code{\link{multilayer_to_json}}, \code{\link{plot_multilayer}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/matrix_to_list_bipartite.R
\name{matrix_to_list_bipartite}
\alias{matrix_to_list_bipartite}
\title{Incidence matrix to edge list conversion}
\usage{
matrix_to_list_bipartite(x, group_names = c("set_cols", "set_rows"))
}
\arguments{
\item{x}{An incidence matrix.}

\item{group_names}{Name of the groups in the columns and rows, respectively (e.g., parasites and hosts).}
}
\value{
A \code{monolayer} object.
}
\description{
Convert an incidence matrix to an edge list.
}
\details{
Assigns column and row names if those do not exist.
Cannot handle node attributes/metadata. Always assumes an undirected network.
}
\examples{

# See examples in: https://emln.ecomplab.com/monolayer.html#Bipartite

matrix_to_list_bipartite(bipartite::memmott1999, group_names = c('Animals', 'Plants'))

}
\seealso{
\code{create_monolayer_network, monolayer}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_igraph.R
\name{get_igraph}
\alias{get_igraph}
\title{Convert a multilayer class to a a list of igraph objects}
\usage{
get_igraph(multilayer, bipartite, directed)
}
\arguments{
\item{multilayer}{The multilayer object.}

\item{bipartite}{Is the network bipartite? Must be provided (no default
value).}

\item{directed}{Is the network directed? Must be provided (no default value).}
}
\value{
A list:
\itemize{
  \item \code{layers_igraph} A list of igraph objects, one per layer
  \item \code{nodes} A table of physical nodes, as in the multlayer object.
  \item \code{state_nodes_map} A table with state nodes to map the nodes to the row and columns of a SAM.
}
}
\description{
This function creates a list of igraph objects from the network layers. Interlayer links are not included.
}
\details{
If link, physical node or state node attributes are included in the multilayer network they are passed to the igraph objects.
}
\examples{

# See examples in:
# https://emln.ecomplab.com/multilayer.html#To_igraph_objects

}

\name{monolayer class}
\alias{monolayer}
\title{An object of class monolayer}

\value{
A list with:
\itemize{
  \item mode: bipartite (B) or unipartite (U)
  \item directed: TRUE/FALSE
  \item nodes: a tibble with information on the nodes (name, id, type, other attributes)
  \item mat: the network in a matrix format
  \item edge_list: the network in an edge list format
  \item igraph_object: the igraph object of the network
}
}
\description{
A network object of class monolayer contains all the necessary information and R objects to perform future analyses on a monolayer network, and in particular analyses using Infomap.
}
\seealso{
\code{list_to_matrix, matrix_to_list_unipartite, matrix_to_list_bipartite, create_network_object}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/list_to_matrix.R
\name{list_to_matrix}
\alias{list_to_matrix}
\title{Edge list to matrix conversion}
\usage{
list_to_matrix(
  x,
  directed = FALSE,
  bipartite = TRUE,
  group_names = c("set_cols", "set_rows"),
  node_metadata = NULL
)
}
\arguments{
\item{x}{An edge list of the format \code{from to weight}.}

\item{directed}{Is the network directed?}

\item{bipartite}{Is network bipartite?}

\item{group_names}{For bipartite networks: name of the groups in the columns
and rows, respectively (e.g., parasites and hosts).}

\item{node_metadata}{Following the igraph method of \code{graph_from_data_frame}.
Must have a column called node_name with names matching those in x.}
}
\value{
A \code{monolayer} object.
}
\description{
Convert an edge list to a matrix. Can handle unipartite and bipartite
networks.
}
\details{
It is also possible to add node attributes. For this, it uses
igraph. The first column in the \code{node_metadata} data frame should have
the names of the nodes in the edge list.
}
\examples{
library(dplyr)
network <- load_emln(17)
nodes <- network$nodes
links<-subset(network$extended_ids \%>\% filter(layer_from==1), select=c(node_from, node_to, weight))

list_to_matrix(links, directed = TRUE, bipartite = FALSE, node_metadata = nodes)


# See examples in:
# https://emln.ecomplab.com/monolayer.html#Unipartite

}
\seealso{
Functions \code{create_monolayer_network, monolayer} and \code{graph_from_data_frame} from igraph.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in
%   R/emln71_temporal_perturbation_kaiser_bunbury_2009.R
\docType{data}
\name{emln71_temporal_perturbation_kaiser_bunbury_2009}
\alias{emln71_temporal_perturbation_kaiser_bunbury_2009}
\title{emln71_temporal_perturbation_kaiser_bunbury_2009}
\source{
Interaction_Web_DataBase

Web_of_Life

http://www.ecologia.ib.usp.br/iwdb/html/kaiser-bunbury_2009.html

http://www.web-of-life.es/map.php
}
\description{
Network Description:\tabular{cccc}{
   network_id \tab multilayer_network_type \tab ecological_network_type \tab state_nodes \cr
   71 \tab Temporal    , Perturbation \tab Pollination \tab FALSE \cr
}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/emln58_spatial_temporal_martins_2020.R
\docType{data}
\name{emln58_spatial_temporal_martins_2020}
\alias{emln58_spatial_temporal_martins_2020}
\title{emln58_spatial_temporal_martins_2020}
\source{
Dryad

https://doi.org/10.5061/dryad.vmcvdncq0
}
\description{
Network Description:\tabular{cccc}{
   network_id \tab multilayer_network_type \tab ecological_network_type \tab state_nodes \cr
   58 \tab Spatial , Temporal \tab Plant-Herbivore \tab TRUE \cr
}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/emln6_environment_kohler_2011.R
\docType{data}
\name{emln6_environment_kohler_2011}
\alias{emln6_environment_kohler_2011}
\title{emln6_environment_kohler_2011}
\source{
Web_of_Life

Interaction_Web_DataBase

http://www.web-of-life.es/map.php?type=6

http://www.ecologia.ib.usp.br/iwdb/html/schleuning-et-al-2010.html
}
\description{
Network Description:\tabular{cccc}{
   network_id \tab multilayer_network_type \tab ecological_network_type \tab state_nodes \cr
   6 \tab Environment \tab Pollination \tab TRUE \cr
}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_sam.R
\name{get_sam}
\alias{get_sam}
\title{Convert a multilayer class to a supra-adjacency matrix}
\usage{
get_sam(
  multilayer,
  bipartite,
  directed,
  sparse = FALSE,
  remove_zero_rows_cols = FALSE
)
}
\arguments{
\item{multilayer}{The multilayer object.}

\item{bipartite}{Is the network bipartite? Must be provided (no default
value).}

\item{directed}{Is the network directed? Must be provided (no default value).}

\item{sparse}{Should the returned matrix be a sparse matrix (class Matrix)?
Defaults to false.}

\item{remove_zero_rows_cols}{Should rows and columns that sum to 0 be
removed? Defaults to false.}
}
\value{
A list:
\itemize{
  \item \code{M} The SAM. Can also be sparse.
  \item \code{nodes} A table of physical nodes, as in the multlayer object.
  \item \code{state_nodes_map} A table with state nodes to map the nodes to the row and columns of the SAM.
}
}
\description{
This function creates a supra-adjacency matrix from a multilayer object.
}
\details{
By default, a SAM contains all the nodes in all the layers. However,
  not all nodes always occur in all layers. This will be reflected in the
  \code{state_nodes_map}: When layer and node ids are NA that means that the
  node did not occur in the layer.

  A node may not have links across all
  the layers. In that case, its row or column sum will be zero. This can happen in a
  directed matrix (see toy example for unipartite network), or in
  diagonally-coupled networks (see toy bipartite example). The user can choose to remove rows and columns that sum to zero by setting \code{remove_zero_rows_cols} to \code{TRUE}.
}
\examples{

# See examples in:
# https://emln.ecomplab.com/multilayer.html#To_supra-adjacency_matrices

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/emln78_temporal_zackenberg_2013.R
\docType{data}
\name{emln78_temporal_zackenberg_2013}
\alias{emln78_temporal_zackenberg_2013}
\title{emln78_temporal_zackenberg_2013}
\source{
plos

https://journals.plos.org/plosone/article?id=10.1371/journal.pone.0081694
}
\description{
Network Description:\tabular{cccc}{
   network_id \tab multilayer_network_type \tab ecological_network_type \tab state_nodes \cr
   78 \tab Temporal \tab Pollination \tab FALSE \cr
}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in
%   R/emln48_environment_perturbation_bartomeus_2008.R
\docType{data}
\name{emln48_environment_perturbation_bartomeus_2008}
\alias{emln48_environment_perturbation_bartomeus_2008}
\title{emln48_environment_perturbation_bartomeus_2008}
\source{
Web_of_Life

Interaction_Web_DataBase

http://www.ecologia.ib.usp.br/iwdb/html/bartomeus_2008.html

http://www.web-of-life.es/map.php
}
\description{
Network Description:\tabular{cccc}{
   network_id \tab multilayer_network_type \tab ecological_network_type \tab state_nodes \cr
   48 \tab Environment , Perturbation \tab Pollination \tab FALSE \cr
}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/emln57_spatial_temporal_kolpelke_2017.R
\docType{data}
\name{emln57_spatial_temporal_kolpelke_2017}
\alias{emln57_spatial_temporal_kolpelke_2017}
\title{emln57_spatial_temporal_kolpelke_2017}
\source{
mangal

https://mangal.io/doc/api/
}
\description{
Network Description:\tabular{ccccc}{
   network_id \tab multilayer_network_type \tab mangal_code \tab ecological_network_type \tab state_nodes \cr
   57 \tab Spatial , Temporal \tab 6 \tab Host-Parasite \tab FALSE \cr
}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/emln.R
\docType{package}
\name{emln-package}
\alias{emln}
\alias{emln-package}
\title{emln: Organize, Handle, and Explore Ecological Multilayer Networks}
\description{
Data and analysis of ecological multilayer networks, including standardization of data structures and functions to convert between them. Includes an interactive multilayer network visualizer (beta, paper forthcoming), and a collection of 78 empirical ecological multilayer network datasets. This work was supported by research grant ISF (Israel Science Foundation) 1281/20 to Shai Pilosof. Noa Frydman (2023) \doi{10.1111/2041-210X.14225}.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://emln.ecomplab.com/}
  \item Report bugs at \url{https://github.com/Ecological-Complexity-Lab/emln/issues}
}

}
\author{
\strong{Maintainer}: Geut Galai \email{geutg@post.bgu.ac.il}

Authors:
\itemize{
  \item Noa Frydman
  \item Shai Pilosof \email{pilos@bgu.ac.il} [funder]
  \item Shirly Freilikhman
}

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/multilayer_to_json.R
\name{multilayer_to_json}
\alias{multilayer_to_json}
\title{Export a multilayer network to JSON}
\usage{
multilayer_to_json(
  multilayer,
  file = NULL,
  bipartite = FALSE,
  setA_type = NULL,
  directed = NULL,
  directed_interlayer = NULL
)
}
\arguments{
\item{multilayer}{A multilayer object (created by \code{create_multilayer_network}
or \code{load_emln}).}

\item{file}{Optional file path to write the JSON to. If NULL, returns the JSON
string.}

\item{bipartite}{Logical. Whether the network is bipartite. Defaults to
\code{FALSE}. MiRA does not auto-detect bipartite structure — you must
set this to \code{TRUE} explicitly. When \code{TRUE}, the nodes table
must contain a \code{node_type} (or legacy \code{node_group}) column
with exactly two distinct values across the network. Any \code{node_group}
column is renamed to \code{node_type} in the output regardless of the
bipartite flag (MiRA's canonical name).}

\item{setA_type}{Optional character. The \code{node_type} value to render
as Set A (top row) in MiRA's bipartite layout. By ecological convention
this is the higher trophic level (e.g. \code{"pollinator"},
\code{"parasite"}, \code{"disperser"}). Only used when
\code{bipartite = TRUE}. If \code{NULL} (default), MiRA falls back to
alphabetical ordering of the two types.}

\item{directed}{Logical or NULL. Whether intralayer links are directed.
If NULL (default), auto-detected by checking intralayer edge symmetry.}

\item{directed_interlayer}{Logical or NULL. Whether interlayer links are
directed. If NULL (default), inherits from \code{directed} (matching
MiRA's parser behavior). Set explicitly when intra- and interlayer
directionality differ.}
}
\value{
If \code{file} is NULL, returns the JSON string invisibly.
  If \code{file} is specified, writes JSON to disk and returns the file path
  invisibly.
}
\description{
Converts an EMLN \code{multilayer} object to JSON format compatible with
MiRA, the multilayer interactive R-network app.
}
\details{
The JSON output contains four arrays matching MiRA's expected format:
\itemize{
  \item \code{nodes}: Physical nodes with \code{node_id}, \code{node_name}, and
    any extra attributes. For bipartite networks, \code{node_group} is mapped to
    \code{node_type}.
  \item \code{layers}: Layer metadata with \code{layer_id}, \code{layer_name},
    and extra attributes. For bipartite networks, a \code{bipartite: true} flag
    is added; if \code{setA_type} is given it is also written per layer.
  \item \code{extended}: The extended edge list with \code{layer_from},
    \code{node_from}, \code{layer_to}, \code{node_to}, \code{weight}, and any
    link attributes.
  \item \code{state_nodes}: State node map with \code{layer_id}, \code{node_id},
    \code{layer_name}, \code{node_name}, plus any extra per-(layer, node)
    attributes (e.g. \code{abundance}, \code{module}).
}
Top-level flags \code{directed} and (when set) \code{directed_interlayer}
indicate edge directionality globally; they are not repeated per link.
}
\note{
MiRA (Multilayer Interactive Rendering Application) is a browser-based
interactive visualization tool for multilayer networks, available at
\url{https://mira.ecomplab.com/}.

If you use MiRA in published research, please cite:

Nehoray SM, Bloch Y, Pilosof S (2026). Interactively visualizing biological
multilayer networks using MiRA. \emph{arXiv}:2605.09597 [cs.SI].
\doi{doi:10.48550/arXiv.2605.09597}

Feedback and bug reports:
\url{https://github.com/Ecological-Complexity-Lab/MiRA/issues}.
}
\examples{
\donttest{
# Export to file
net <- load_emln(14)
multilayer_to_json(net, file = "tests/my_network.json", bipartite = TRUE)

# Bipartite with explicit Set A ordering
multilayer_to_json(net, bipartite = TRUE, setA_type = "pollinator")

# Get JSON string
json_str <- multilayer_to_json(net)
}
}
\seealso{
\code{plot_multilayer, create_multilayer_network, load_emln}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/loadRData.R
\name{loadRData}
\alias{loadRData}
\title{Internal function - loads an RData file, and returns it}
\usage{
loadRData(fileName)
}
\arguments{
\item{fileName}{file to be downloaded}
}
\description{
Internal function - loads an RData file, and returns it
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{descriptions}
\alias{descriptions}
\title{View the data sets}
\format{
An object of class \code{tbl_df} (inherits from \code{tbl}, \code{data.frame}) with 89 rows and 10 columns.
}
\usage{
descriptions
}
\description{
@format Data set of multilayer networks
}
\details{
@source \code{create}

 @examples
 data(descriptions)
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/emln36_spatial_joern_1979.R
\docType{data}
\name{emln36_spatial_joern_1979}
\alias{emln36_spatial_joern_1979}
\title{emln36_spatial_joern_1979}
\source{
Web_of_Life

Interaction_Web_DataBase

http://www.web-of-life.es/map.php?type=6

http://www.ecologia.ib.usp.br/iwdb/html/joern_1979.htm
}
\description{
Network Description:\tabular{cccc}{
   network_id \tab multilayer_network_type \tab ecological_network_type \tab state_nodes \cr
   36 \tab Spatial \tab Plant-Herbivore \tab TRUE \cr
}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_multilayer.R
\name{.resolve_mira_path}
\alias{.resolve_mira_path}
\title{Resolve path to the bundled MiRA web app}
\usage{
.resolve_mira_path(mira_path = NULL)
}
\description{
Looks first for an installed copy at
\code{system.file("MiRA", package = "emln")}, then falls back to
sibling/dev locations for in-source development.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/search_emln.R
\name{search_emln}
\alias{search_emln}
\title{Search networks}
\usage{
search_emln(
  ecological_network_type = NULL,
  multilayer_network_type = NULL,
  state_nodes = NULL,
  node_number_minimum = NULL,
  layer_number_minimum = NULL,
  weighted = NULL,
  interlayer = NULL,
  directed = NULL
)
}
\arguments{
\item{ecological_network_type}{One of: Seed-Dispersal, Pollination, Food-Web, Host-Parasite, Multiple, Plant-Herbivore, Anemone-Fish, Plant-Ant}

\item{multilayer_network_type}{Environment, Temporal, Multiplex, Perturbation, Spatial}

\item{state_nodes}{TRUE/FALSE}

\item{node_number_minimum}{The minimum number of layers wanted in the network}

\item{layer_number_minimum}{The minimum number of layers wanted in the network}

\item{weighted}{TRUE/FALSE}

\item{interlayer}{TRUE/FALSE}

\item{directed}{TRUE/FALSE}
}
\value{
A tibble containing the network id, and all the search arguments for each of the networks that are compatible with the search.
}
\description{
Search the database for networks depending on search parameters specified below.
}
\examples{
# See examples in:
# https://emln.ecomplab.com/data.html#Browsing_the_data

# Generates a tibble of all the networks in the package that are pollination networks
search_emln(ecological_network_type = 'Plant-Ant')

# Generates a tibble of all the networks in the package that have state nodes and are temporal
search_emln(multilayer_network_type = 'Temporal', state_node = TRUE)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/emln73_temporal_ramos_robles_2016.R
\docType{data}
\name{emln73_temporal_ramos_robles_2016}
\alias{emln73_temporal_ramos_robles_2016}
\title{emln73_temporal_ramos_robles_2016}
\source{
PeerJ

https://peerj.com/articles/2048/#supplemental-information
}
\description{
Network Description:\tabular{cccc}{
   network_id \tab multilayer_network_type \tab ecological_network_type \tab state_nodes \cr
   73 \tab Temporal \tab Seed-Dispersal \tab FALSE \cr
}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/emln26_spatial_chacoff_2020.R
\docType{data}
\name{emln26_spatial_chacoff_2020}
\alias{emln26_spatial_chacoff_2020}
\title{emln26_spatial_chacoff_2020}
\source{
Dryad

https://datadryad.org/stash/dataset/doi:10.5061/dryad.8cz8w9gm1
}
\description{
Network Description:\tabular{cccc}{
   network_id \tab multilayer_network_type \tab ecological_network_type \tab state_nodes \cr
   26 \tab Spatial \tab Pollination \tab FALSE \cr
}
}
\keyword{internal}
